const _ = require('lodash');
const axios = require('axios');
const colors = require('colors'); // eslint-disable-line no-unused-vars
const args = require('yargs')
    .demandOption(['clientId', 'clientSecret', 'domain', 'usersFile'])
    .argv;
const fs = require('fs');

axios.defaults.baseURL = `https://${args.domain}`;

const usersJson = JSON.parse(fs.readFileSync(args.usersFile, 'utf8'));

axios.interceptors.response.use(undefined, (err) => {
    if (err.response.status === 429 && err.config && (!err.config.__retryCount || err.config.__retryCount < 3)) {
        err.config.__retryCount = err.config.__retryCount ? err.config.__retryCount + 1 : 1;
        return new Promise((resolve, reject) => {
            console.log("Call failed with 429 - retrying in 6000ms");
            setTimeout(() => {
                axios(err.config).then(resolve, reject);
            }, 6000)
        });
    }
    throw err;
});

axios.post("/oauth/token", {
    "grant_type":"client_credentials",
    "client_id": args.clientId,
    "client_secret": args.clientSecret,
    "audience": `https://${args.domain}/api/v2/`
}).then(({data: {access_token: accessToken}}) => {
    axios.defaults.headers.common['Authorization'] = `Bearer ${accessToken}`;

    const recur = (page, prev) => {
        return axios.get(`/api/v2/users?per_page=100&fields=email,user_id&page=${page}&search_engine=v3`).then(({ data }) => {
            const list = prev.concat(data);
            if (data.length < 100) {
                return list;
            } else {
                return recur(page + 1, list);
            }
        });
    };

    function generatePassword() {
        var length = 12,
            charset = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789",
            retVal = "";
        for (var i = 0, n = charset.length; i < length; ++i) {
            retVal += charset.charAt(Math.floor(Math.random() * n));
        }
        return "Aa1" + retVal;
    }

    recur(0, []).then(list => {
        const usersByEmail = _.groupBy(list, "email");
        const chunks = _.chunk(usersJson, 5);

        console.log(`Processing ${chunks.length} chunks`);

        const promises = chunks.map((users, index) => {
            return new Promise((resolve) => {
                setTimeout(() => {
                    console.log(`Next chunk: ${index}`);
                    Promise.all(users.map((user) => {
                        const existing = usersByEmail[user.email];
                        if(existing){
                            console.log(existing);
                            return Promise.resolve({
                                email: user.email,
                                id: existing[0].user_id,
                                existing: true
                            })
                        } else {
                            return axios.post(`/api/v2/users`, {
                                connection: "Username-Password-Authentication",
                                email: user.email,
                                password: generatePassword(),
                                verify_email: false,
                                "user_metadata": {
                                    name: user.name,
                                    company: user.company
                                },
                                "app_metadata": {
                                    migrated: "cloud-sync"
                                }
                            }).then(({data}) => ({
                                email: user.email,
                                id: data.user_id,
                                existing: false
                            }), err => ({
                                email: user.email,
                                error: _.get(err, "response.data.message") || err.message
                            }));
                        }
                    })).then(resolve);
                }, index*10000);
            });
        });

        Promise.all(promises).then(list => {
            fs.writeFileSync("out.json", JSON.stringify(list));
            console.log(`Output written to out.json`);
        });
    })
}).catch(err => {
    console.log("Unable to finish porting users..".red, err);
    process.exit(1);
});