const fs = require('fs');
const fse = require('fs-extra');
const { execSync } = require('child_process');

const buildDir = "build";

console.log("Removing previous build directory");
fse.removeSync(buildDir);

console.log("Create new build directory");
fs.mkdirSync(buildDir);

console.log("Moving source files to build directory");
fse.copySync("src/", buildDir, { filter: src => !src.endsWith("test.js") });

console.log("Moving yarn file to build directory");
fse.copySync("yarn.lock", "build/yarn.lock");

console.log("Copying user-admin database spec files");
fse.copySync("../auth0-account/src/connections/", `${buildDir}/connections/`);

console.log("Copying package.json and serverless files");
fse.copySync("package.json", "build/package.json");
fse.copySync("cloud-central.yml", "build/cloud-central.yml");
fse.copySync("subscription-service.yml", "build/subscription-service.yml");

console.log("Installing production dependencies");
execSync('yarn install --production', { cwd: 'build', stdio: 'inherit' });

console.log("Removing build file from build directory");
fse.removeSync(`${buildDir}/package.json`);

console.log("Moving node_modules into layer");
fse.moveSync(`${buildDir}/node_modules`, `${buildDir}/nodejs/node_modules`);

console.log("Packaging serverless - cloud central");
fse.copySync("build/cloud-central.yml", "build/serverless.yml");
execSync(`serverless package --stage ${process.env.stage} --package cloud-central`, { cwd: 'build', stdio: 'inherit' });

console.log("Packaging serverless - subscription");
fse.copySync("build/subscription-service.yml", "build/serverless.yml");
execSync(`serverless package --stage ${process.env.stage} --package subscription-service`, { cwd: 'build', stdio: 'inherit' });
