const util = require('util');
const AWS = require('aws-sdk');

AWS.config.update({ region: 'us-east-1' });
AWS.config.setPromisesDependency(require('bluebird'));


exports.assessmentFinished = async ({ userId, body }) => {
    console.log(`Received assessment results for ${userId} with value ${util.inspect(body, { showHidden: false, depth: null })}`);

    const dynamoDb = new AWS.DynamoDB.DocumentClient();

    const createParams = {
        TableName: process.env.cloudAssessmentReportTable,
        Item: {
            userId,
            reportTime: body.reportTime,
            body
        }
    };

    return dynamoDb.put(createParams).promise().catch(err => {
        console.log(`Failed saving assessment results to dynamo. ${err.message} ..`);
        console.log(err);
        return null;
    }).then(() => null);
};

exports.assessmentFailed = async ({ userId, body }) => {
    console.log(`Assessment failed for ${userId} with value ${util.inspect(body, { showHidden: false, depth: null })}`);

    return Promise.resolve(null);
};
