const axios = require("axios");
const _ = require("lodash");
const { sendToKinesisLogStream } = require("./utils/logging");

const authAccount = exports.authAccount = {
    domain: process.env.managementDomain,
    godClientId: process.env.managementClientId,
    godClientSecret: process.env.managementClientSecret
};

exports.authAxios = _.once(() => {
    const instance = axios.create({
        baseURL: `https://${authAccount.domain}/`
    });

    const audience = `https://${authAccount.domain}/api/v2/`;

    instance.interceptors.response.use(response => {
        const kinesisInput = {
            type: "AUTH-RATE",
            timestamp: new Date().valueOf(),
            data: {
                "x-ratelimit-limit": _.get(response, "headers.x-ratelimit-limit"),
                "x-ratelimit-remaining": _.get(response, "headers.x-ratelimit-remaining"),
                "x-ratelimit-reset": _.get(response, "headers.x-ratelimit-reset"),
                url: _.get(response, "config.url")
            }
        };

        return sendToKinesisLogStream({ data: kinesisInput }).catch(err => {
            console.log("Failed to log auth0 rate data");
            console.log(err);
            return null;
        }).then(() => response);
    }, err => {
        if (err.statusCode === 429 && err.config && (!err.config.__retryCount || err.config.__retryCount < 3)) {
            err.config.__retryCount = err.config.__retryCount ? err.config.__retryCount + 1 : 1;
            return new Promise((resolve, reject) => {
                console.log("Call failed with 429 - retrying in 3000ms");
                setTimeout(() => {
                    axios(err.config).then(resolve, reject);
                }, 3000);
            });
        }
        throw err;
    });

    return instance.post(`oauth/token`, {
        grant_type: "client_credentials",
        client_id: authAccount.godClientId,
        client_secret: authAccount.godClientSecret,
        audience
    }).then(({ data: { access_token: accessToken } }) => {
        instance.defaults.headers.common.Authorization = `Bearer ${accessToken}`;
        instance.defaults.baseURL = audience;
        return instance;
    }, err => {
        console.error(`Failed to retrieve oath token for domain: ${authAccount.domain} with clientId: ${authAccount.godClientId}.`, err);
        throw new Error("Failed retrieving management credentials.");
    });
});
