const _ = require('lodash');

const ClientError = exports.ClientError = class ClientError extends Error {
    constructor(...args) {
        super(...args);
        Error.captureStackTrace(this, ClientError);
        this.name = this.constructor.name;
    }
};

const ResourceNotFoundError = exports.ResourceNotFoundError = class ResourceNotFoundError extends Error {
    constructor(...args) {
        super(...args);
        Error.captureStackTrace(this, ResourceNotFoundError);
        this.name = this.constructor.name;
    }
};

const ForbiddenError = exports.ForbiddenError = class ForbiddenError extends Error {
    constructor(...args) {
        super(...args);
        Error.captureStackTrace(this, ForbiddenError);
        this.name = this.constructor.name;
    }
};

const MethodNotAllowedError = exports.MethodNotAllowedError = class MethodNotAllowedError extends Error {
    constructor(...args) {
        super(...args);
        Error.captureStackTrace(this, MethodNotAllowedError);
        this.name = this.constructor.name;
    }
};

exports.mapsErrors = err => {
    const mapped = {
        statusCode: 500,
        response: {
            message: "Unknown server exception"
        }
    };

    if (err instanceof ClientError) {
        mapped.statusCode = 400;
        mapped.response.message = err.message;
    } else if (err instanceof ResourceNotFoundError) {
        mapped.statusCode = 404;
        mapped.response.message = "Not Found";
    } else if (err instanceof ForbiddenError) {
        mapped.statusCode = 403;
        mapped.response.message = "Forbidden";
    } else if (err instanceof MethodNotAllowedError) {
        mapped.statusCode = 405;
        mapped.response.message = "Method Not Allowed";
    } else {
        mapped.response.message = _.get(err, 'response.data.message') || err.message;
    }

    return mapped;
};
