const AWS = require('aws-sdk');

AWS.config.update({ region: 'us-east-1' });
AWS.config.setPromisesDependency(null);
const dynamoDb = new AWS.DynamoDB.DocumentClient();

const eventMatcher = event => {
    switch (event.type) {
        case "CREATE-RELATIONSHIP": {
            if (!event.userId) {
                return Promise.reject(new Error('No userId present for CREATE-RELATIONSHIP event'));
            }

            const params = {
                TableName: process.env.userInfoTable,
                Key: {
                    userId: `${event.userId}`
                },
                UpdateExpression: "set hasCSRelationship = :val",
                ExpressionAttributeValues: {
                    ":val": true
                }
            };

            return dynamoDb.update(params).promise().catch(err => {
                console.log("Failed updating flag in dynamodb");
                console.log(err);
                throw new Error("Failed updating CREATE-RELATIONSHIP event in database");
            });
        }
        default:
            return Promise.reject(event.type ? new Error(`No match for event type ${event.type}`) : "No event type specified.");
    }
};

exports.handler = (event, context, callback) => {
    console.log("Received push event from cloud sync");
    console.log(event);

    eventMatcher(event).then(response => callback(null, response), err => callback(err));
};


const yarger = require("yargs") // eslint-disable-line no-unused-vars, import/order
    .command("test", "test", yargs => {
        yargs.demandOption(["userId", "type"]);
    }, argv => {
        process.env.userInfoTable = "portal-backend-dev-userInfo";
        eventMatcher({ userId: argv.userId, type: argv.type });
    }).argv;
