const { formatLogsForElasticSearch } = require("./auth0-logs-fetcher");

let defaultLogEntry;
let defaultResponse;
const generateResponseWithValues = values => ({
    groups: {
        "auth-undefined-2019.1.1": [{
            ...defaultResponse,
            ...values,
            id: 1
        }]
    },
    lastEntryId: 1
});

beforeEach(() => {
    defaultLogEntry = { _id: 1, date: 1546300800000, type: 'ssa' };
    defaultResponse = {
        auth_type: "ssa",
        auth_type_text: "Success Silent Auth",
        date: 1546300800000,
        id: 1
    };
});

describe("formatLogsForElasticSearch", () => {
    it("should work for simple case", () => {
        const promise = formatLogsForElasticSearch([defaultLogEntry]);
        return expect(promise).resolves.toEqual(generateResponseWithValues());
    });

    describe('handling error payload', () => {
        it("should handle object error payload ", () => {
            const promise = formatLogsForElasticSearch([{
                ...defaultLogEntry,
                details: { error: { payload: { key: "I am a value" } } }
            }]);
            return expect(promise).resolves.toEqual(generateResponseWithValues({
                details: { error: { payload: { key: "I am a value" } } }
            }));
        });

        it("should handle object as a string error payload ", () => {
            const promise = formatLogsForElasticSearch([{
                ...defaultLogEntry,
                details: { error: { payload: '{"key": "I am a value"}' } }
            }]);
            return expect(promise).resolves.toEqual(generateResponseWithValues({
                details: { error: { payload: { key: "I am a value" } } }
            }));
        });

        it("should handle non object error payload ", () => {
            const promise = formatLogsForElasticSearch([{
                ...defaultLogEntry,
                details: { error: { payload: 'Not an object' } }
            }]);
            return expect(promise).resolves.toEqual(generateResponseWithValues({
                details: { error: { payload: { stringifiedPayload: 'Not an object' } } }
            }));
        });
    });

    describe('handling scope permissions', () => {
        it('should handle cc:* scope permissions', () => {
            const promise = formatLogsForElasticSearch([{
                ...defaultLogEntry,
                details: { request: { body: { app_metadata: { "cc:permission": true } } } }
            }]);
            return expect(promise).resolves.toEqual(generateResponseWithValues({
                details: { request: { body: { app_metadata: { "cc:permission": true } } } }
            }));
        });

        it('should handle cc:* scope permissions which are not boolean', () => {
            const promise = formatLogsForElasticSearch([{
                ...defaultLogEntry,
                details: {
                    request: { body: { app_metadata: { "cc:permission": 'I am a typo' } } },
                    response: { body: { app_metadata: { "cc:permission": 'I am a typo' } } }
                }
            }]);
            return expect(promise).resolves.toEqual(generateResponseWithValues({
                details: {
                    request: { body: { app_metadata: { "_illegalKey_cc:permission": '"I am a typo"' } } },
                    response: { body: { app_metadata: { "_illegalKey_cc:permission": '"I am a typo"' } } }
                }
            }));
        });
    });
});
