const { authAxios } = require("./../auth-utils");
const { sendInternalEmail } = require('./internal-email');
const { internalDevTeamMailList } = require("./../consts");
const { blacklist } = require("./../consts");

const enableMFAForEmailDomain = domain => {
    console.log(`Attempting to enable mfa for the domain: ${domain}`);
    if (blacklist.includes(domain)) {
        console.log(`Cannot enable MFA for blacklisted domain ${domain}. Exiting`);
        return Promise.reject(new Error(`Cannot enable MFA for blacklisted domain ${domain}. Exiting`));
    }
    const configKey = `mfa_${domain.replace(/\./g, '_')}`;

    return authAxios().then(client => client.put(`rules-configs/${configKey}`, { value: "true" })
        .then(() => {
            const subject = `Enabled MFA for domain ${domain}`;
            const message = `Enabled MFA in environment: ${process.env.stage} <br /> for email domain: ${domain}`;
            return sendInternalEmail(subject, message, internalDevTeamMailList).then(() => {
                console.log(`Successfully sent internal email`);
                return `mfa successfully enabled`;
            }, err => {
                console.log(`Failed to send internal email: ${err}`);
                return `mfa successfully enabled`;
            });
        }).catch(err => {
            console.error(`Error. Failed to enable mfa`, err);
            throw err;
        }));
};

const disableMFAForEmailDomain = domain => {
    console.log(`Attempting to disable mfa for the domain: ${domain}`);
    const configKey = `mfa_${domain.replace(/\./g, '_')}`;

    return authAxios().then(client => client.delete(`rules-configs/${configKey}`)
        .then(() => `mfa successfully disabled`).catch(err => {
            console.error(`Error. Failed to disable mfa`, err);
            throw err;
        }));
};

/*eslint-disable */
const yargs = require("yargs")
    .command("enable-mfa", "enable-mfa", yargs => {
        yargs.demandOption(["emailDomain"]);
    }, argv => {
        if (!process.env.managementDomain) { throw new Error("No management settings in variables"); }
        if (!argv.emailDomain) { throw new Error("Missing required parameter emailDomain"); }
        enableMFAForEmailDomain(argv.emailDomain).then(res => console.log(res), err => process.exit(1));
    })
    .command("disable-mfa", "disable-mfa", yargs => {
        yargs.demandOption(["emailDomain"]);
    }, argv => {
        if (!process.env.managementDomain) { throw new Error("No management settings in variables"); }
        if (!argv.emailDomain) { throw new Error("Missing required parameter emailDomain"); }
        disableMFAForEmailDomain(argv.emailDomain).then(res => console.log(res), err => process.exit(1));
    }).argv;
/* eslint-enable */
