const _ = require("lodash");
const { validateHourlyChargeRequest } = require("../services/validator");
const { processTimestamp } = require("../utils");
const { totalPerSubscription } = require("../services/hourlyCharge");
const { getLogger } = require("../logger");

const logger = getLogger();

const validateRequestBody = (req, res, next) => {
    logger.info(`VALIDATING REQUEST BODY`);
    logger.info(JSON.stringify(req.body));
    validateHourlyChargeRequest(req.body)
        .then(() => {
            logger.info(`Successfully validated request body`);
            next();
        })
        .catch(err => {
            next(`request body validation error: ${err}`);
        });
};

const addMergedId = (req, res, next) => {
    logger.info(`ADDING MERGED ID TO CHARGES`);
    _.forEach(req.body, charge => { charge.mergedId = `${charge.subscriptionId}-${charge.dimension}`; });
    next();
};

const validateUniqueness = (req, res, next) => {
    logger.info(`VALIDATING CHARGES UNIQUENESS`);
    const origNumCharges = req.body.length;
    const comparator = (chargeVal, otherChargeVal) => ((chargeVal.mergedId === otherChargeVal.mergedId) && (processTimestamp(chargeVal.unixTimestamp, "hourly") === processTimestamp(otherChargeVal.unixTimestamp, "hourly")));
    req.body = _.uniqWith(req.body, comparator);
    if (req.body.length === origNumCharges) {
        logger.info(`Successfully validated charges uniqueness`);
    } else {
        logger.warning(`Duplicated items identified`);
    }
    next();
};

// TODO: test
const calculateTotals = (req, res, next) => {
    logger.info(`CALCULATING TOTALS`);
    _.forEach(req.body, charge => {
        const accountsTotals = totalPerSubscription(charge.accounts);
        logger.info(`Totals: ${JSON.stringify(accountsTotals)}`);
        charge.accumulatedVal = accountsTotals.accumulatedVal;
        charge.accounts = accountsTotals.accumulatedAccounts;
    });
    next();
};

exports.validateRequestBody = validateRequestBody;
exports.addMergedId = addMergedId;
exports.validateUniqueness = validateUniqueness;
exports.calculateTotals = calculateTotals;
