const AWS = require('aws-sdk/index');
const _ = require("lodash");
const { doWithRetry } = require("../utils");

const { getLogger } = require("../logger");

const logger = getLogger();
AWS.config.setPromisesDependency(require('bluebird'));

const marketplaceMetering = new AWS.MarketplaceMetering({ region: 'us-east-1', accessKeyId: process.env.sellerAccessKey, secretAccessKey: process.env.sellerSecretKey });

// const commonResponses = [
//     "TimestampOutOfBoundsException: Timestamp 2019-04-29T16:43:43.834Z is out of allowed range",
//     "InvalidCustomerIdentifierException: An invalid customer identifier was specified: YhRWuVafgdg",
//     "InvalidParameterType: Expected params.UsageRecords[0].Timestamp to be a Date object, ISO-8601 string, or a UNIX timestamp",
//     "Success",
//     "DuplicateRecord",
//     "CustomerNotSubscribed"
// ];

const shouldRetry = ({ code, statusCode }, retries) => (
    code === "ThrottlingException"
    || statusCode == 500 // eslint-disable-line eqeqeq
    || statusCode == 503 // eslint-disable-line eqeqeq
) && (retries < process.env.retries);

const reportMetering = (productCode, usageRecords, retries = 0) => {
    const params = {
        ProductCode: productCode,
        UsageRecords: usageRecords
    };
    return doWithRetry(shouldRetry)(0, () => marketplaceMetering.batchMeterUsage(params).promise())
        .then(response => {
            if (!_.isEmpty(response.UnprocessedRecords)) {
                logger.warn(`Not all batch was processed, retrying again`);
                if (retries >= process.env.retries) {
                    logger.error(`!!!CRITICAL!!! No more retries. Failed to process batch`);
                    throw new Error(`!!!CRITICAL!!! No more retries. Failed to process batch`);
                }
                return reportMetering(productCode, usageRecords, retries + 1);
            }
            return response;
        });
};

module.exports.reportMetering = reportMetering;
