const Joi = require("joi");
const { getLogger } = require("../logger");
const accumulatedChargePerAccountSchema = require("../models/accumulatedChargePerAccount");
const hourlyChargeSchema = require("../models/hourlyCharges");
const resolvedHourlyChargeSchema = require("../models/reportedHourlyCharges");
const usagesPerProductSchema = require("../models/usagesPerProduct");

const logger = getLogger();

exports.validateHourlyChargeRequest = requestBody => new Promise((resolve, reject) => {
    Joi.validate(requestBody, hourlyChargeSchema, err => {
        if (err) {
            logger.error(err);
            reject(err);
        }
        resolve(true);
    });
});

exports.validateResolvedHourlyChargeRequest = requestBody => new Promise((resolve, reject) => {
    Joi.validate(requestBody, resolvedHourlyChargeSchema, err => {
        if (err) {
            logger.error(err);
            reject(err);
        }
        resolve(true);
    });
});


exports.validateAccumulatedChargePerAccountSchema = accumulatedCharges => new Promise((resolve, reject) => {
    Joi.validate(accumulatedCharges, accumulatedChargePerAccountSchema, err => {
        if (err) {
            logger.error(err);
            reject(err);
        }
        resolve(true);
    });
});

exports.validateUsagesPerProduct = usagesPerProduct => new Promise((resolve, reject) => {
    Joi.validate(usagesPerProduct, usagesPerProductSchema, err => {
        if (err) {
            logger.error(err);
            reject(err);
        }
        resolve(true);
    });
});
