const moment = require("moment/moment");
const Promise = require("bluebird");
const { getLogger } = require("./logger");

const logger = getLogger();

exports.promiseInspect = promise => promise.then(
    value => ({ value, status: "fulfilled" }),
    error => ({ error, status: "rejected" })
);

exports.processTimestamp = (isoString, reduceTo = "daily") => {
    const momentWithEpoch = moment(isoString).utc();
    if (reduceTo === "daily") {
        momentWithEpoch.startOf("day");
    } else {
        momentWithEpoch.startOf("hour");
    }
    return momentWithEpoch.valueOf();
};

exports.fromUnixToIsoString = unixTimestamp => moment(unixTimestamp).utc().toISOString(true);

const doWithRetry = shouldRetry => (retries, asyncFunc) => asyncFunc()
    .catch(error => {
        const retry = shouldRetry(error, retries);
        logger.warn(`Got an error: ${error} \nNumber or retries: ${retries} \nNeed to retry? ${retry}`);
        if (!retry) {
            return Promise.reject(error);
        }
        return Promise.delay(process.env.intervalMillisec * (retries + 1))
            .then(() => doWithRetry(shouldRetry)(retries + 1, asyncFunc));
    });

exports.doWithRetry = doWithRetry;
