const { CanvasRenderService } = require('chartjs-node-canvas');
const { createCanvas, loadImage } = require('canvas')
const AWS = require('aws-sdk');
const util = require('util');
const fs = require('fs');
const dateFormat = require('dateformat');
const { Parser } = require('json2csv');
var nodemailer = require('nodemailer');

AWS.config.update({ 
    region: 'us-east-1'
 });
//TODO
// 1 : function that recives dates and return the chart data
// 2 : split to API and schdualed fuction

// eslint-disable-line import/no-extraneous-dependencies


//Consts
const isSendMailEnable = process.env.sendMail;
const isStoreS3Enable = process.env.storeS3;
const mailList = process.env.mailList;

const monthsList = ["January","February","March","April","May","June","July","August","September","October","November","December"]
const regionsMaping = [
    {
        name:"global",
        x:65,
        y:571
    },
    {
        name:"NoRegion",
        x:80,
        y:591
    },
    {
        name:"us-east-2",
        x:257,
        y:250
    },
    {
        name:"us-east-1",
        x:286,
        y:274
    },
    {
        name:"us-west-1",
        x:127,
        y:290
    },
    {
        name:"us-west-2",
        x:116,
        y:245
    },
    {
        name:"ap-east-1",
        x:1060,
        y:320
    },
    {
        name:"ap-south-1",
        x:905,
        y:345
    },
    {
        name:"ap-northeast-2",
        x:1090,
        y:286
    },
    {
        name:"ap-southeast-1",
        x:1032,
        y:394
    },
    {
        name:"ap-southeast-2",
        x:1203,
        y:528
    },
    {
        name:"ap-northeast-1",
        x:1154,
        y:263
    },
    {
        name:"ca-central-1",
        x:309,
        y:228
    },
    {
        name:"cn-north-1",
        x:1063,
        y:243
    },
    {
        name:"cn-northwest-1",
        x:1025,
        y:278
    },
    {
        name:"eu-central-1",
        x:631,
        y:235
    },
    {
        name:"eu-west-1",
        x:567,
        y:193
    },
    {
        name:"eu-west-2",
        x:590,
        y:216
    },
    {
        name:"eu-west-3",
        x:600,
        y:260
    },
    {
        name:"eu-north-1",
        x:654,
        y:186
    },
    {
        name:"sa-east-1",
        x:394,
        y:482
    },
    {
        name:"us-gov-east-1",
        x:232,
        y:292
    },
    {
        name:"us-gov-west-1",
        x:120,
        y:260
    }
]
const fileNames = { chartByAccount: "chartByAccount.png",csvByAccount:"CsvByAccount.csv",csvByRegion:"CsvByRegion.csv",costDistributionMap:"costDistributionMa.png"}

const mailOptions = {
    from: 'ng-tlv-eng-devops@netapp.com',
    subject: 'Cost History by Months and Accounts',
    html: `<h1>Cost History by Months and Accounts</h1><img src="`+fileNames.chartByAccount+`">`,
    to: mailList,
    // bcc: Any BCC address you want here in an array,
    
};

const today = new Date();

const reportDatesByAccount = {
    startDate : dateFormat(new Date(today.getFullYear() - 1,today.getMonth(),today.getDate()),"isoDate"),
    endDate: dateFormat(today,"isoDate")
};

//TODO: make configerble
const reportDatesByRegion = {
    startDate : dateFormat(new Date(today.getFullYear(),today.getMonth(),1),"isoDate"),
    endDate: dateFormat(today,"isoDate")
};

const tempAccountString = "Sasha,718273455463,#EC7063;Lian,052582346341,#9B59B6;E2E,282316784512,#3498DB;OCCM_Dev,733004784675,#1ABC9C;Hili,620697567579,#4286f4;E2EHA,800707617106,#F9E79F;XCP,464262061435,#EB984E;Dekel,582244788873,#ef1fe5;OCCM_QA,642991768967,#979A9A;Chen,872751033342,#1C2833;Ishai,459776933209,#D5DBDB;Cloud_Portal,055518636490,#78281F;DFIO,167338112540,#1B4F72;Demo,759995470648,#A2D9CE;OCCM,952013314444,#ABB2B9;VPN,741989522154,#AED6F1";
const global_accounts = parseRoles(tempAccountString);


function createSESService(){
    return new AWS.SES();
}

function createCostService(roleName,roleArn) {
    //const sts = new AWS.STS();
    const sts_params = {
        RoleArn: roleArn,
        RoleSessionName: roleName
    };

    return new AWS.CostExplorer({ region: 'us-east-1', credentials: new AWS.TemporaryCredentials({RoleArn: sts_params.RoleArn}) })
    
}



// bug workaround: https://github.com/vmpowerio/chartjs-node/issues/26
if (global.CanvasGradient === undefined) {
    global.CanvasGradient = function() {};
}

async function renderMap(Data){
    const canvas = createCanvas(1300, 600)
    const ctx = canvas.getContext('2d')

    const myimg = await loadImage('map.png').then((image) => {
        ctx.drawImage(image, 0, 0, 1300, 600)

        Data.forEach(function(elem){
            const mapDetails = regionsMaping.find(obj => obj.name === elem.Region);
            ctx.fillText("$"+elem.Amount+" - "+elem.Region, mapDetails.x, mapDetails.y)
        })
        

        return canvas.toBuffer();
        
    })
    return myimg;
    
    
}

async function renderChartWithChartJS(labels, Data,costs) {
    const width = 1300;
    const height = 1000;
    const axesData = Data.map((currElement, index) => {
		const account = {}
		account.label = currElement.name
		account.data = costs[index]
		account.backgroundColor = currElement.color
		return account
    });
    const configuration = {
        type: 'bar',
        data: {
            labels,
            datasets: axesData
        },
        label: "AWS Monthly Costs",
        options: {
            scales: {
                xAxes: [{
                    scaleLabel:{
                        display:true,
                        labelString:"Months",
                    },
                    ticks: {
                        // Include a dollar sign in the ticks
                        callback: function(value, index, values) {
                            const date = new Date(value);
                            return monthsList[date.getMonth()]+"-"+date.getFullYear();
                        }
                    
                    },
                    stacked: true,
                }],
                yAxes: [{
                    scaleLabel:{
                        display:true,
                        labelString:"Cost in US Dollars ($)",
                    },
                    ticks: {
                        beginAtZero: true,
                        // Include a dollar sign in the ticks
                        callback: function(value, index, values) {
                            return value?parseFloat(value)/1000+'K':value;
                        }
                    
                    },
                    stacked: true
            }
                ]
            },
            legend: {
                display: true,
                labels: {
                    fontColor: 'rgb(255, 99, 132)'
                }
            },
            title: {
                display: true,
                text: 'AWS Monthly Costs'
            }
        }
    };
    const chartCallback = (ChartJS) => {
        ChartJS.defaults.global.responsive = true;
        ChartJS.defaults.global.maintainAspectRatio = false;
    };

    const canvasRenderService = new CanvasRenderService(width, height, chartCallback);
    return await canvasRenderService.renderToBuffer(configuration);
}

const writeResultsToBucket = async (name,buffer) => {
    console.log(`Creating object in S3`);
    const s3 = new AWS.S3();

    const bucketParams = {
        Bucket: process.env.privateBucket,
        Body: buffer,
        Key: 'maintenance/'+name
    };
    return s3.putObject(bucketParams).promise();

};

function parseRoles(accounts){
    const accountsStringArray = accounts.split(';');
    const accountsArray = [];
    accountsStringArray.map(accountstring => {const account = accountstring.split(','); accountsArray.push({name:account[0],role:"arn:aws:iam::"+account[1]+":role/RoleForCost",color:account[2]})})
    return accountsArray
};

const getMonthlyCostByAccount = async (costExplorer) => {
     
    const costParams = {
        TimePeriod: {
            Start: reportDatesByAccount.startDate.toString(),
            End: reportDatesByAccount.endDate.toString()
        },
        Granularity: "MONTHLY",
        Metrics: [
            "UnblendedCost"
        ]
    };
    const ans = (await costExplorer.getCostAndUsage(costParams).promise()).ResultsByTime;
    return ans;
};

const getMonthlyCostByRegion = async (costExplorer) => {
    
    const costParams = {
        TimePeriod: {
            Start: reportDatesByRegion.startDate.toString(),
            End: reportDatesByRegion.endDate.toString()
        },
        Granularity: "MONTHLY",
        Metrics: [
            "UnblendedCost"
        ],
        GroupBy: [
        {
            Key: "REGION",
            Type: "DIMENSION"
        }]
    };
    const ans = (await costExplorer.getCostAndUsage(costParams).promise()).ResultsByTime;
    return ans;
};

function accountsDataToCsvTable(accounts,labels,data){
    const dataTable = accounts.map((currElement,index)=> 
    {
        const row = {}
        row["Account"] = currElement;
        for (let i = 0; i < labels.length; i++) {
            row[labels[i]] =  data[index][i]            
        }
        return row;
    })
    const sum = {
        Account:'Sum'
    }
    for (let i = 0; i < labels.length; i++) {
        sum[labels[i]] =  data.map(account => account[i]).reduce((a,b) => parseFloat(a) + parseFloat(b)).toFixed(0)         
    }
    return dataTable.concat(sum)
}

function regionDataToCsv(RegionDictionary){
    const dataJson = []
    for (const [key, value] of Object.entries(RegionDictionary)) {

        dataJson.push({Region:key,Amount:value.toFixed(2)})
    }
    dataJson.push({Region:"Sum",Amount:dataJson.map(obj=>obj.Amount).reduce((a,b) => parseFloat(a) + parseFloat(b)).toFixed(0)})
    return dataJson;
}

function sendMailWithAttachments(callback) {
    const ses = createSESService();
    var transporter = nodemailer.createTransport({
        SES: ses
    });
    transporter.sendMail(mailOptions, function (err, info) {
        if (err) {
            console.log(err);
            console.log('Error sending email');
            callback(err);
        }
        else {
            console.log('Email sent successfully');
            callback();
        }
    });
}

exports.handler = async (event, context, callback) => {
    try{
        console.log("Start\r\n");

        //Creating cost eplorer service for each role
        const costexplorerArr = global_accounts.map(account => createCostService(account.name,account.role));

        // GET cost by account bar chart
        const costsArrayByAccount = await  Promise.all(costexplorerArr.map((element) => getMonthlyCostByAccount(element)));
        const costsByAccount = costsArrayByAccount.map(elem => elem.map(data => parseFloat(data.Total.UnblendedCost.Amount).toFixed(2)))
        const labelsByAccount = costsArrayByAccount[0].map(data => data.TimePeriod.Start);
        //Render chart
        const bufferByAccount = await renderChartWithChartJS(labelsByAccount,global_accounts, costsByAccount);
        

        //Create cost by account CSV
        const csvByAccount = accountsDataToCsvTable(global_accounts.map(account=>account.name),labelsByAccount.map(month => monthsList[new Date(month).getMonth()]),costsByAccount)
        const fieldsByAccount = ['Account'];
        const headersByAccount = fieldsByAccount.concat(labelsByAccount.map(month => monthsList[new Date(month).getMonth()]))
        const optsByAccount = { headersByAccount };
        const parserByAccount = new Parser(optsByAccount);
        const csvByAccountParsed = parserByAccount.parse(csvByAccount);
        
        

        //Get cost by region 
        const costsArrayByRegion = await  Promise.all(costexplorerArr.map((element) => getMonthlyCostByRegion(element)));
        //TODO: data.Keys[0]????
        let costByRegionDic = {};
        costsArrayByRegion.map(elem => elem.map(groups => groups.Groups.map(data=>{
            costByRegionDic[data.Keys[0]] = costByRegionDic[data.Keys[0]]?(parseFloat(costByRegionDic[data.Keys[0]]) + parseFloat(data.Metrics.UnblendedCost.Amount)):parseFloat(data.Metrics.UnblendedCost.Amount)
        })));


        //Create CSV
        const csvByRegion = regionDataToCsv(costByRegionDic);
        const fieldsByRegion = ['Region','Amount'];
        const optsByRegion = { fieldsByRegion };
        const parserByRegion = new Parser(optsByRegion);
        const csvByRegionParsed = parserByRegion.parse(csvByRegion);
        
        
        //Render Map
        const mapData = csvByRegion.slice(0, -1);
        const renderedMap = await renderMap(mapData);

        if(isStoreS3Enable){
            await writeResultsToBucket(fileNames.chartByAccount,bufferByAccount);
            await writeResultsToBucket(fileNames.csvByAccount,csvByAccountParsed);
            await writeResultsToBucket(fileNames.csvByRegion,csvByRegionParsed);
            await writeResultsToBucket(fileNames.costDistributionMap,renderedMap);
        }

        if(isSendMailEnable){
            //Send mail
            mailOptions.attachments = [
                {
                    filename: fileNames.chartByAccount,
                    content: bufferByAccount,
                    cid: fileNames.chartByAccount
                },{
                    filename: fileNames.csvByAccount,
                    content: csvByAccountParsed
                },{
                    filename: fileNames.csvByRegion,
                    content: csvByRegionParsed
                },{
                    filename: fileNames.costDistributionMap,
                    content: renderedMap
                }
            ]
            sendMailWithAttachments(callback);
        }
        

    }
    catch(e){
        return callback(e)
    }
    callback(null, {})
};


