import React from 'react';
import { storiesOf } from '@storybook/react';
import { MemoryRouter } from 'react-router';
import buttons from '../../ui-components/src/button/button.module.scss';
import dialogs from "../../ui-components/src/dialog/dialog.module.scss";
import styles from './components.module.scss';
import { FormFieldInput } from "ui-components";
import { ReactComponent as InlineLoader } from "ui-components/src/images/loaders/inline-loader.svg";
import list from "ui-components/src/list/list.module.scss";

const ButtonView = (props = { children, title }) => (
    <div className={styles.item}>
        <p className={styles.title}>{props.title}</p>
        {props.children}
    </div>
);

storiesOf('Components', module)
    .addDecorator(story => (
        <MemoryRouter initialEntries={['/']}>{story()}</MemoryRouter>
    ))
    .add('Buttons and Dialogs', () => (
        <div className={styles.wrapper}>
            <ButtonView title="Regular">
                <button className={buttons.primary}>Idle</button>
            </ButtonView>
            <ButtonView title="Regular Disabled">
                <button className={buttons.primary} disabled={true}>Disabled</button>
            </ButtonView>
            <ButtonView title="Cancel">
                <button className={buttons.cancel}>Cancel</button>
            </ButtonView>
            <ButtonView title="Dangerous Action">
                <button className={buttons.dangerous}>Danger</button>
            </ButtonView>
        </div>
    ))
    .add('Dialogs', () => (
        <div style={{
            display: "flex",
            flexDirection: "column",
            width: "400px",
            border: "2px solid black",
            padding: "20px"
        }}>
            <p className={styles.title}>Button Pair for Dialogs</p>
            <div className={dialogs["button-pair-wrapper"]}>
                <button className={buttons.primary}>Idle</button>
                <button className={buttons.cancel}>Cancel</button>
            </div>
        </div>
    ))
    .add('Inline Loader', () => (
        <div>
            <InlineLoader className="inline-loader"/>
        </div>
    ))
    .add('Loading Mask', () => (
        <div>
            <div className={styles['loading-mask']}/>
        </div>
    ))
    .add('Input', () => (
        <div className={styles["input-wrapper"]}>
            <FormFieldInput label="Empty" type="text" placeholder="placeholder text" autoFocus={true}/>
            <FormFieldInput label="With Value" type="text" value="User text"/>
            <FormFieldInput label="Pending" type="text" value="Think about that" pending={true} InlineLoaderIcon={InlineLoader}/>
            <FormFieldInput label="With Error" type="text" value="Wrong value" errorMessage="Something is wrong."/>
            <FormFieldInput label="Another one" type="text"/>
        </div>
    ))
    .add('List', () => (
        <div style={{fontSize: "15px", padding: "20px"}}>
            <span style={{fontWeight: "bold", display: "inline-block"}}>unordered list with discs</span>
            <ul className={list["disc-ul"]}>
                <li>"At vero eos et accusamus et iusto odio dignissimos ducimus qui blanditiis praesentium voluptatum deleniti atque corrupti quos dolores et quas molestias excepturi sint occaecati cupiditate non provident, similique sunt in culpa qui officia deserunt mollitia animi, id est laborum et dolorum fuga. Et harum quidem rerum facilis est et expedita distinctio. Nam libero tempore, cum soluta nobis est eligendi optio cumque nihil impedit quo minus id quod maxime placeat facere possimus, omnis voluptas assumenda est, omnis dolor repellendus. Temporibus autem quibusdam et aut officiis debitis aut rerum necessitatibus saepe eveniet ut et voluptates repudiandae sint et molestiae non recusandae. Itaque earum rerum hic tenetur a sapiente delectus, ut aut reiciendis voluptatibus maiores alias consequatur aut perferendis doloribus asperiores repellat."</li>
                <li>"Sed ut perspiciatis unde omnis iste natus error sit voluptatem accusantium doloremque laudantium, totam rem aperiam, eaque ipsa quae ab illo inventore veritatis et quasi architecto beatae vitae dicta sunt explicabo. Nemo enim ipsam voluptatem quia voluptas sit aspernatur aut odit aut fugit, sed quia consequuntur magni dolores eos qui ratione voluptatem sequi nesciunt. Neque porro quisquam est, qui dolorem ipsum quia dolor sit amet, consectetur, adipisci velit, sed quia non numquam eius modi tempora incidunt ut labore et dolore magnam aliquam quaerat voluptatem. Ut enim ad minima veniam, quis nostrum exercitationem ullam corporis suscipit laboriosam, nisi ut aliquid ex ea commodi consequatur? Quis autem vel eum iure reprehenderit qui in ea voluptate velit esse quam nihil molestiae consequatur, vel illum qui dolorem eum fugiat quo voluptas nulla pariatur?"</li>
                <li>List Item 3</li>
                <li>List Item 4</li>
            </ul>

            <span style={{fontWeight: "bold", display: "inline-block", marginTop: "8px"}}>unordered list with circle</span>
            <ul className={list["circle-ul"]}>
                <li>"At vero eos et accusamus et iusto odio dignissimos ducimus qui blanditiis praesentium voluptatum deleniti atque corrupti quos dolores et quas molestias excepturi sint occaecati cupiditate non provident, similique sunt in culpa qui officia deserunt mollitia animi, id est laborum et dolorum fuga. Et harum quidem rerum facilis est et expedita distinctio. Nam libero tempore, cum soluta nobis est eligendi optio cumque nihil impedit quo minus id quod maxime placeat facere possimus, omnis voluptas assumenda est, omnis dolor repellendus. Temporibus autem quibusdam et aut officiis debitis aut rerum necessitatibus saepe eveniet ut et voluptates repudiandae sint et molestiae non recusandae. Itaque earum rerum hic tenetur a sapiente delectus, ut aut reiciendis voluptatibus maiores alias consequatur aut perferendis doloribus asperiores repellat."</li>
                <li>"Sed ut perspiciatis unde omnis iste natus error sit voluptatem accusantium doloremque laudantium, totam rem aperiam, eaque ipsa quae ab illo inventore veritatis et quasi architecto beatae vitae dicta sunt explicabo. Nemo enim ipsam voluptatem quia voluptas sit aspernatur aut odit aut fugit, sed quia consequuntur magni dolores eos qui ratione voluptatem sequi nesciunt. Neque porro quisquam est, qui dolorem ipsum quia dolor sit amet, consectetur, adipisci velit, sed quia non numquam eius modi tempora incidunt ut labore et dolore magnam aliquam quaerat voluptatem. Ut enim ad minima veniam, quis nostrum exercitationem ullam corporis suscipit laboriosam, nisi ut aliquid ex ea commodi consequatur? Quis autem vel eum iure reprehenderit qui in ea voluptate velit esse quam nihil molestiae consequatur, vel illum qui dolorem eum fugiat quo voluptas nulla pariatur?"</li>
                <li>List Item 3</li>
                <li>List Item 4</li>
            </ul>

            <span style={{fontWeight: "bold", display: "inline-block", marginTop: "8px"}}>unordered list with chevron image</span>
            <ul className={list["chevron-ul"]}>
                <li>"At vero eos et accusamus et iusto odio dignissimos ducimus qui blanditiis praesentium voluptatum deleniti atque corrupti quos dolores et quas molestias excepturi sint occaecati cupiditate non provident, similique sunt in culpa qui officia deserunt mollitia animi, id est laborum et dolorum fuga. Et harum quidem rerum facilis est et expedita distinctio. Nam libero tempore, cum soluta nobis est eligendi optio cumque nihil impedit quo minus id quod maxime placeat facere possimus, omnis voluptas assumenda est, omnis dolor repellendus. Temporibus autem quibusdam et aut officiis debitis aut rerum necessitatibus saepe eveniet ut et voluptates repudiandae sint et molestiae non recusandae. Itaque earum rerum hic tenetur a sapiente delectus, ut aut reiciendis voluptatibus maiores alias consequatur aut perferendis doloribus asperiores repellat."</li>
                <li>"Sed ut perspiciatis unde omnis iste natus error sit voluptatem accusantium doloremque laudantium, totam rem aperiam, eaque ipsa quae ab illo inventore veritatis et quasi architecto beatae vitae dicta sunt explicabo. Nemo enim ipsam voluptatem quia voluptas sit aspernatur aut odit aut fugit, sed quia consequuntur magni dolores eos qui ratione voluptatem sequi nesciunt. Neque porro quisquam est, qui dolorem ipsum quia dolor sit amet, consectetur, adipisci velit, sed quia non numquam eius modi tempora incidunt ut labore et dolore magnam aliquam quaerat voluptatem. Ut enim ad minima veniam, quis nostrum exercitationem ullam corporis suscipit laboriosam, nisi ut aliquid ex ea commodi consequatur? Quis autem vel eum iure reprehenderit qui in ea voluptate velit esse quam nihil molestiae consequatur, vel illum qui dolorem eum fugiat quo voluptas nulla pariatur?"</li>
                <li>List Item 3</li>
                <li>List Item 4</li>
            </ul>

            <span style={{fontWeight: "bold", display: "inline-block", marginTop: "8px"}}>ordered list</span>
            <ol className={list["number-ol"]}>
                <li>"At vero eos et accusamus et iusto odio dignissimos ducimus qui blanditiis praesentium voluptatum deleniti atque corrupti quos dolores et quas molestias excepturi sint occaecati cupiditate non provident, similique sunt in culpa qui officia deserunt mollitia animi, id est laborum et dolorum fuga. Et harum quidem rerum facilis est et expedita distinctio. Nam libero tempore, cum soluta nobis est eligendi optio cumque nihil impedit quo minus id quod maxime placeat facere possimus, omnis voluptas assumenda est, omnis dolor repellendus. Temporibus autem quibusdam et aut officiis debitis aut rerum necessitatibus saepe eveniet ut et voluptates repudiandae sint et molestiae non recusandae. Itaque earum rerum hic tenetur a sapiente delectus, ut aut reiciendis voluptatibus maiores alias consequatur aut perferendis doloribus asperiores repellat."</li>
                <li>"Sed ut perspiciatis unde omnis iste natus error sit voluptatem accusantium doloremque laudantium, totam rem aperiam, eaque ipsa quae ab illo inventore veritatis et quasi architecto beatae vitae dicta sunt explicabo. Nemo enim ipsam voluptatem quia voluptas sit aspernatur aut odit aut fugit, sed quia consequuntur magni dolores eos qui ratione voluptatem sequi nesciunt. Neque porro quisquam est, qui dolorem ipsum quia dolor sit amet, consectetur, adipisci velit, sed quia non numquam eius modi tempora incidunt ut labore et dolore magnam aliquam quaerat voluptatem. Ut enim ad minima veniam, quis nostrum exercitationem ullam corporis suscipit laboriosam, nisi ut aliquid ex ea commodi consequatur? Quis autem vel eum iure reprehenderit qui in ea voluptate velit esse quam nihil molestiae consequatur, vel illum qui dolorem eum fugiat quo voluptas nulla pariatur?"</li>
                <li>List Item 3</li>
                <li>List Item 4</li>
            </ol>
        </div>

    ))
    .add('Select', () => (
        <div>TBD</div>
    ));