import React from 'react';
import { storiesOf } from '@storybook/react';
import {Link} from 'react-router-dom';
import { MemoryRouter } from 'react-router';
import { ReactComponent as NetAppIcon } from 'ui-components/src/images/netapp-services/ic-netapp.svg';
import { ReactComponent as CloudVolumesLogo } from "ui-components/src/images/netapp-services/ic-cvs.svg";
import { ReactComponent as CloudSyncLogo } from 'ui-components/src/images/netapp-services/ic-cloud-sync.svg';

import '../../ui/src/components/internal/SalesToolsView.scss';
import cards from '../../ui-components/src/card/card.module.scss';
import cssGrid from "ui-components/src/grid/grid.module.scss";
import classNames from "classnames";


storiesOf('Internal Pages', module)
    .addDecorator(story => (
        <MemoryRouter initialEntries={['/']}>{story()}</MemoryRouter>
    ))
    .add('h4', () => (
        <div className="internal-sales-view" style={{padding: 0}}>
            <h4>Header 4</h4>
        </div>
    ))
    .add('Cards grid arrangement', () => (
        <div style={{width: 1135, margin: 'auto'}}>
            <h2 style={{color: "red"}}>To see the effects of large screen decrease the sidebar's size</h2>
            <h3>1 Cards in a Row</h3>
            <div className={classNames(cssGrid["col-1-sm"], cssGrid["col-1-lg"])}>
                <Link className={cards.card} to="">
                    <NetAppIcon/>
                    <div>Card 1</div>
                </Link>
            </div>
            <h3>2 Cards in a Row</h3>
            <div className={classNames(cssGrid["col-1-sm"], cssGrid["col-2-lg"])}>
                <Link className={cards.card} to="">
                    <NetAppIcon/>
                    <div>Card 1</div>
                </Link>
                <Link className={cards.card} to="">
                    <NetAppIcon/>
                    <div>Card 2</div>
                </Link>
                <Link className={cards.card} to="">
                    <NetAppIcon/>
                    <div>Card 3</div>
                </Link>
            </div>
            <h3>3 Cards in a Row</h3>
            <div className={classNames(cssGrid["col-1-sm"], cssGrid["col-3-lg"])}>
                <Link className={cards.card} to="">
                    <NetAppIcon/>
                    <div>Card 1</div>
                </Link>
                <Link className={cards.card} to="">
                    <NetAppIcon/>
                    <div>Card 2</div>
                </Link>
                <Link className={cards.card} to="">
                    <NetAppIcon/>
                    <div>Card 3</div>
                </Link>
                <Link className={cards.card} to="">
                    <NetAppIcon/>
                    <div>Card 4</div>
                </Link>
            </div>
            <h3>4 Cards in a Row</h3>
            <div className={classNames(cssGrid["col-1-sm"], cssGrid["col-4-lg"])}>
                <Link className={cards.card} to="">
                    <NetAppIcon/>
                    <div>Card 1</div>
                </Link>
                <Link className={cards.card} to="">
                    <NetAppIcon/>
                    <div>Card 2</div>
                </Link>
                <Link className={cards.card} to="">
                    <NetAppIcon/>
                    <div>Card 3</div>
                </Link>
                <Link className={cards.card} to="">
                    <NetAppIcon/>
                    <div>Card 4</div>
                </Link>
                <Link className={cards.card} to="">
                    <NetAppIcon/>
                    <div>Card 5</div>
                </Link>
            </div>
        </div>
    ))
    .add('Card types', () => (
        <div style={{width: 1135, margin: 'auto'}}>
            <h3>Standard Card</h3>
            <Link className={cards.card} to="">
                <NetAppIcon/>
                <div>Card 1</div>
            </Link>
            <h3>Service Cards</h3>
            <div className={classNames(cssGrid["col-1-sm"], cssGrid["col-2-lg"])}>
                <Link className={cards.service} to="">
                    <CloudSyncLogo/>
                    <div>Card 1</div>
                </Link>
                <Link className={cards.storage} to="">
                    <CloudVolumesLogo/>
                    <div>Card 1</div>
                </Link>
            </div>
            <h3>Info Cards</h3>
            <div className={classNames(cssGrid["col-1-sm"], cssGrid["col-2-lg"])}>
                <Link className={cards.info} to="">
                    <div className={cards.title}>Title</div>
                    <div className={cards.keywords}>Keyword 1 | keyword 2</div>
                    <p className={cards.details}>
                        What if we told you that NetApp now offers native cloud volume What if we told you that NetApp now offers native cloud volume
                    </p>
                </Link>
                <Link className={cards.info} to="">
                    <div className={cards.title}>Title</div>
                    <div className={cards.keywords}>Keyword 1 | keyword 2</div>
                    <div className={cards.details}>
                        What if we told you that NetApp now offers native cloud volume What if we told you that NetApp now offers native cloud volume
                    </div>
                </Link>
            </div>
        </div>
    ));

