const axios = require('axios');
const Promise = require('bluebird');
const Initializer = require('../utils/specInitializer');
const Constants = require('../utils/Constants');

const { logger, clients } = Initializer('OCCM cleanup script');
const password = "Netapp234!";

const deleteUserCloudSync = (token) => {
    return axios.delete(`${Constants.cloudSyncApiBaseUrl}/users`, {
        headers: {
            Authorization: `Bearer ${token}`
        }
    });
};

const emailToDelete = () => {
    const mailRegex = new RegExp(`(${Constants.testEmail})[0-9]+(@netapp-test.com)`, 'i');
    return clients.auth0Client.getUsers('email')
        .then(users => {
            return users.map(user => user.email);
        })
        .then(emails => {
            return emails.filter(email => {
                return mailRegex.test(email);
            });
        });
};

const deleteUsers = () => {
    const deletedUsers = [];
    const notDeletedCloudSync = [];
    const notDeletedCloudCentral = [];
    return emailToDelete()
        .then(emailsToDelete => {
            logger.info(`Emails for deletion: ${JSON.stringify(emailsToDelete)}`);
            logger.info(`Number of emails for deletion: ${emailsToDelete.length}`);
            return Promise.map(emailsToDelete, async (email) => {
                try {
                    try {
                        if(process.env.domain !== "dev") {
                            const cloudSyncToken = await clients.auth0Client.getCloudSyncUserToken(email, password);
                            await deleteUserCloudSync(cloudSyncToken);
                        }
                    } catch (error) {
                        if(error.response.data) {
                            logger.error(JSON.stringify(error.response.data));
                        }
                        logger.error(`Could not delete ${email} from cloud sync`);
                        notDeletedCloudSync.push(email);
                    }
                    const govToken = await clients.auth0Client.getGovernanceUserToken(email, password);
                    await clients.governanceClient.deleteAssociatedOccms(govToken);
                    await clients.governanceClient.deleteUser(govToken);
                    deletedUsers.push(email);
                } catch (error) {
                    if(error.response.data) {
                        logger.error(JSON.stringify(error.response.data));
                    }
                    logger.error(`Could not delete ${email} from cloud central`);
                    notDeletedCloudCentral.push(email);
                }
            }, {concurrency: 1})
                .then(() => {
                    logger.info(`Deleted users: ${JSON.stringify(deletedUsers)}`);
                    logger.info(`Were not deleted from Cloud Sync: ${JSON.stringify(notDeletedCloudSync)}`);
                    logger.info(`Were not deleted from Cloud Central: ${JSON.stringify(notDeletedCloudCentral)}`);
                    logger.info(`# of deleted users: ${deletedUsers.length}`);
                    logger.info(`# that were not deleted from Cloud Sync: ${notDeletedCloudSync.length}`);
                    logger.info(`# that were not deleted from Cloud Central: ${notDeletedCloudCentral.length}`);
                });
        });
};

deleteUsers().catch(console.log);

