const _ = require('lodash');
const axios = require('axios');
const Constants = require('../Constants');
const {resolvedResponseInterceptor, rejectedResponseInterceptor, resolvedRequestInterceptor, rejectedRequestInterceptor} = require('../axios-response-handler');

exports.authManagementAxios = _.once(() => {
    const instance = axios.create({
        baseURL: Constants.auth0BaseUrl
    });
    instance.interceptors.request.use(resolvedRequestInterceptor, rejectedRequestInterceptor);
    instance.interceptors.response.use(resolvedResponseInterceptor, rejectedResponseInterceptor);
    return instance.post(`oauth/token`, {
        "grant_type":"client_credentials",
        "client_id": Constants.auth0ManagementClientId,
        "client_secret": Constants.auth0ManagementClientSecrete,
        "audience": `${Constants.auth0BaseUrl}/api/v2/`
    }).then(response => {
        instance.defaults.headers.common['Authorization'] = `Bearer ${response.access_token}`;
        instance.defaults.baseURL = `${Constants.auth0BaseUrl}/api/v2/`;
        return instance;
    })
});

exports.authAxios = _.once(() => {
    const instance = axios.create({
        baseURL: Constants.auth0BaseUrl
    });
    instance.interceptors.request.use(resolvedRequestInterceptor, rejectedRequestInterceptor);
    instance.interceptors.response.use(resolvedResponseInterceptor, rejectedResponseInterceptor);
    return instance;
});