const Constants = require('../Constants');
const jwtDecoder = require('../general-utils').decodeToken;
const cloudCentralAxios = require('./cloudCentralAxios').cloudCentralAxios();

class cloudCentralClient {

    constructor(logger) {
        this.logger = logger;
    }

    getCustomerSuccessStories(userToken) {
        this.logger.info(`getting customer success stories data on behalf of ${jwtDecoder(userToken).sub} from ${Constants.cloudCentralApiBaseUrl}/portal/customer-success-stories`);
        return cloudCentralAxios.get(`/portal/customer-success-stories`, {
            headers: {
                Authorization: `Bearer ${userToken}`
            }
        })
    }

    uploadCustomerSuccessStories(userToken, csvFileString) {
        this.logger.info(`Uploading customer success stories file on behalf of ${jwtDecoder(userToken).sub} from ${Constants.cloudCentralApiBaseUrl}/portal/customer-success-stories`);
        return cloudCentralAxios.put(`/portal/customer-success-stories`, {
            content: csvFileString
        }, {
            headers: {
                Authorization: `Bearer ${userToken}`
            }
        });
    }

    updateUserInfo(userToken, userName) {
        this.logger.info(`Updating User name information of ${jwtDecoder(userToken).sub} to ${userName}`);
        return cloudCentralAxios.post(`/portal/update-user-info`, {

            userName
        }, {
            headers: {
                Authorization: `Bearer ${userToken}`
            }
        });
    }

    getUserNotifications(userToken) {
        this.logger.info(`Getting User notifications information of ${jwtDecoder(userToken).sub} `);
        return cloudCentralAxios.get(`/portal/notifications`, {
            headers: {
                Authorization: `Bearer ${userToken}`
            }
        });
    }

    getUsersIdInfo(userToken, usersIdsString) {
        this.logger.info(`Getting Users information of ${jwtDecoder(userToken).sub} `);
        return cloudCentralAxios.get(`/portal/tenancy/users?userIds=${usersIdsString}`,{
            headers: {
                Authorization: `Bearer ${userToken}`
            }
        });
    }
    getUserIdByemail(userToken, userEmail) {
        this.logger.info(`Getting Users information of ${jwtDecoder(userToken).sub} `);
        return cloudCentralAxios.get(`/portal/tenancy/user-id-by-email?email=${userEmail}`,{
            headers: {
                Authorization: `Bearer ${userToken}`
            }
        });
    }
    doesUserExist(userToken, userEmail) {
        this.logger.info(`Getting Users information of ${jwtDecoder(userToken).sub} `);
        return cloudCentralAxios.get(`/portal/does-user-exist?email=${userEmail}`,{
            headers: {
                Authorization: `Bearer ${userToken}`
            }
        });
    }
}

module.exports = cloudCentralClient;
