import axios from "axios";
import {history} from "./../store";
import querystring from "query-string";

export default ({dispatch}) => next => action => {
    if (action.type === "AUTH:REFRESH-SSO") {
        const loginSuccess = (authResult) => {
            axios.defaults.headers.common['Authorization'] = `Bearer ${authResult.accessToken}`;

            dispatch({
                type: "AUTH:REFRESH-SSO-SUCCESS",
                payload: authResult
            });

            setTimeout(() => {
                dispatch({
                    type: "AUTH:REFRESH-SSO",
                });
            }, authResult.expiresIn * 1000 - 10000);

            if(authResult.paramsForUI.pathname){
                const path = authResult.paramsForUI.pathname.startsWith('/') ? authResult.paramsForUI.pathname : '/' + authResult.paramsForUI.pathname;
                history.replace(path);
            } else {
                const search = querystring.parse(window.location.search);
                delete search.redirectToLogin;
                delete search.paramsForLoginPage;
                const searchString = querystring.stringify(search);
                history.replace(window.location.pathname + (searchString ? "?" + searchString : ''));
            }
        };

        const loginFail = (() => {
            dispatch({
                type: "AUTH:REFRESH-SSO-FAILED"
            });
            history.replace(window.location.pathname + window.location.search);
        });

        window.auth.refreshSso().then(loginSuccess, loginFail);
    }

    return next(action);
}