(function () {
    let deferred;
    const openWidget = () => {
        iframe.style.right = 0;
        mask.style.display = 'block';
        setTimeout(() => mask.addEventListener("click", closeWidget));
    };

    const closeWidget = () => {
        iframe.style.right = '-1385px';
        mask.style.display = 'none';
        mask.removeEventListener("click", closeWidget);
    };

    const sendMessage = ({ type, payload }) => {
        window.accountManagementWidget.widgetReady
            .then(() => {
                iframe.contentWindow.postMessage({ type, payload }, '*');
            });
    };

    window.onmessage = function (event) {
        if (event.origin === portalBasePath || event.origin === window.location.origin) {
            const { type, payload } = event.data;

            if (type === 'closeAccountManagementWidget') {
                closeWidget();
            } else if (type === 'accountManagementReady') {
                deferred.resolve();
            } else if (type === 'accountDataChanged') {
                window.accountManagementWidget.accountDataChanged(payload);
            }
        }
    };

    function handleAction(action) {
        if (action.type === 'OPEN-WIDGET') {
            openWidget();
        } else {
            sendMessage(action);
        }
    }

    window.accountManagementWidget = window.accountManagementWidget || {};
    // eslint-disable-next-line lodash/prefer-noop
    window.accountManagementWidget.accountDataChanged = window.accountManagementWidget.accountDataChanged || function () { };

    window.accountManagementWidget.widgetReady = new Promise((resolve, reject) => {
        deferred = { resolve, reject };
    });
    window.accountManagementWidget.sendMessage && window.accountManagementWidget.sendMessage.q && window.accountManagementWidget.sendMessage.q.map(handleAction);
    window.accountManagementWidget.sendMessage = handleAction;

    const portalBasePath = /**PORTALBASEPATH**/window.location.protocol + "//portal.netapp.io:3000";/**PORTALBASEPATH**/

    const loadUrl = portalBasePath + "/components/account";

    const iframe = document.createElement("iframe");
    iframe.id = "accountManagementIframe";
    const mask = document.createElement('div');
    mask.id = "accountManagementIframeMask";

    const loadFrame = () => {
        const maxInt = 2147483647;
        iframe.src = loadUrl;

        iframe.style.height = '100%';
        iframe.style.width = '1358px';
        iframe.style.position = "fixed";
        iframe.style.top = 0;
        iframe.style.right = '-1385px';
        iframe.style.border = 'none';
        iframe.style.transition = 'all 0.4s cubic-bezier(.25, .8, .25, 1)';
        iframe.style.boxShadow = '20px 0 40px 0 #434957';
        iframe.style.zIndex = maxInt;
        document.body.appendChild(iframe);

        mask.style.position = 'fixed';
        mask.style.top = 0;
        mask.style.right = 0;
        mask.style.left = 0;
        mask.style.bottom = 0;
        mask.style.backgroundColor = 'rgba(67, 73, 87, 0.3)';
        mask.style.display = 'none';
        mask.style.zIndex = maxInt - 1;
        document.body.appendChild(mask);
    };

    if (document.readyState === "loading") {
        document.addEventListener("DOMContentLoaded", () => {
            loadFrame();
        });
    } else {
        loadFrame();
    }

}());