import React from 'react';
import _ from "lodash";
import { connect } from 'react-redux';

import styles from "./Accounts.module.scss";
import { ManageAgentWorkspacesOverlay } from "./ManageWorkspacesOverlay";
import TableComponent from "./TableComponent";

const getColumnDefs = () => {
    const fields = [
        {
            headerName: 'Name',
            field: 'name',
            tooltipField: 'name',
            width: 640,
            cellClass: 'one-line-text',
            cellStyle: { 'line-height': `${rowHeight}px`, 'padding-left': '30px' }
        },
        { headerName: 'Workspaces', field: 'workspaces', cellRenderer: 'workspacesCellRenderer', width: 260 }
    ];

    const columnDefs = fields.map(field => _.assign(field, {
        suppressMenu: true,
        unSortIcon: true,
        sortable: true,
        lockPosition: true
    }));
    columnDefs.push({
        headerName: "",
        field: "Name",
        cellRenderer: "addRemoveWorkspaceCellRenderer",
        colId: 'button',
        lockPosition: true,
        cellStyle: { 'justify-content': 'flex-end', 'padding-right': '30px' }
    });

    return columnDefs;
};

const AddRemoveWorkspaceCellRenderer = props => (
    <div key={props.data.name} className={styles["menu-cell"]}>
        <button className={styles["inline-grid-button"]}
                onClick={() => props.context.manageAgentWorkspacesPrompt({ agent: props.data })}>
            Manage Workspaces
        </button>
    </div>
);

const rowHeight = 55; // px

class AgentsTab extends React.Component {
    render() {
        return (<div className={styles["agents-tab"]}>
            {(this.props.fetchAgentsError || this.props.fetchUsersAndWorkspacesError) &&
            <div className={styles["main-view-error-message"]}>{this.props.fetchAgentsError || this.props.fetchUsersAndWorkspacesError}</div>}
            <h4>Account {this.props.account && `${this.props.account.accountName} Agents`}</h4>

            {<TableComponent columnDefs={getColumnDefs()}
                             rowData={this.props.agents}
                             singularTitle="Agent"
                             pluralTitle="Agents"
                             frameworkComponents={{
                                 addRemoveWorkspaceCellRenderer: AddRemoveWorkspaceCellRenderer
                             }}
                             context={{
                                 manageAgentWorkspacesPrompt: this.props.manageAgentWorkspacesPrompt
                             }}/>}
            {this.props.agentForWorkspacesManagement && <ManageAgentWorkspacesOverlay type="Agent"/>}
        </div>);
    }
}

const mapStateToProps = state => ({
    account: state.accountsManagement.account,
    agents: state.accountsManagement.agents,
    agentForWorkspacesManagement: state.accountsManagement.agentForWorkspacesManagement,
    fetchAgentsError: state.accountsManagement.fetchAgentsError,
    fetchUsersAndWorkspacesError: state.accountsManagement.fetchUsersAndWorkspacesError
});

const mapDispatchToProps = dispatch => ({
        manageAgentWorkspacesPrompt: ({ agent }) => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:MANAGE-AGENT-WORKSPACES-PROMPT",
                payload: { agent }
            });
        }
    }
);

export default connect(mapStateToProps, mapDispatchToProps)(AgentsTab);
