import React from 'react';
import { connect } from 'react-redux';
import _ from 'lodash';

import { ReactComponent as SubscriptionIcon } from "ui-components/src/images/monochrome-icons/ic-subscription.svg";
import buttons from "ui-components/src/button/button.module.scss";
import styles from "./Accounts.module.scss";
import Loader from "../../Loader";

const isAccountsEqual = (account1, account2) => account1 && account2 && account1.accountPublicId && account2.accountPublicId && account1.accountPublicId === account2.accountPublicId;

const getAccountsByIds = ({ accountIds, allAccounts }) => _.map(accountIds, accountId => _.find(allAccounts, { accountPublicId: accountId }));

class ManageAccountsOverlay extends React.Component {
    constructor(props) {
        super(props);

        const currentlyAssociatedAccounts = getAccountsByIds({
            accountIds: this.props.entityForAccountsManagement.accounts,
            allAccounts: this.props.allAccounts
        });

        this.state = {
            associatedAccounts: currentlyAssociatedAccounts.map(account => _.assignIn({}, account, { checked: true })),
            unAssociatedAccounts: _.differenceWith(this.props.allAccounts, currentlyAssociatedAccounts, isAccountsEqual),
            changedDone: false
        };
    }

    updateChangesDone() {
        this.setState({
            changesDone: _.some(this.state.associatedAccounts, account => !account.checked) ||
                _.some(this.state.unAssociatedAccounts, 'checked')
        });
    }

    handleCheckboxChange = (accountsGroup, accountPublicId) => {
        this.setState(prevState => ({
            [accountsGroup]: prevState[accountsGroup].map(account => {
                if (account.accountPublicId === accountPublicId) account.checked = !account.checked;
                return account;
            })
        }), this.updateChangesDone);
    };

    submitButtonClicked() {
        const accountIdsToRemove = _(this.state.associatedAccounts).reject('checked').map('accountPublicId').value();
        const accountIdsToAdd = _(this.state.unAssociatedAccounts).filter('checked').map('accountPublicId').value();

        this.props.manageEntityAccounts({
            entity: this.props.entityForAccountsManagement,
            accountIdsToRemove,
            accountIdsToAdd
        });
    }

    render() {
        return (<div className={styles["manage-accounts-overlay"]}>
            {this.props.manageSubscriptionAccountsPending && <Loader/>}
            <SubscriptionIcon/>
            <div className={styles["icon-title"]}>{this.props.entityForAccountsManagement.name}</div>
            <div className={styles["title"]}>Manage the {this.props.type}'s Accounts</div>
            <div className={styles["accounts-list"]}>
                {(this.state.associatedAccounts.length || null) && <h5>Associated Accounts:</h5>}
                {this.state.associatedAccounts.map(account => (
                    <label key={account.accountPublicId} title={account.accountName} style={{ fontWeight: 600 }}>
                        <input type="checkbox"
                               defaultChecked={true}
                               onChange={() => this.handleCheckboxChange('associatedAccounts', account.accountPublicId)}/>
                        <span>{account.accountName}</span>
                    </label>))}
                {(this.state.unAssociatedAccounts.length || null) && (this.state.associatedAccounts.length || null) && <h5>Other Accounts:</h5>}
                {this.state.unAssociatedAccounts.map(account => (
                    <label key={account.accountPublicId} title={account.accountName}>
                        <input type="checkbox" defaultChecked={false}
                               onChange={() => this.handleCheckboxChange('unAssociatedAccounts', account.accountPublicId)}/>
                        <span>{account.accountName}</span>
                    </label>))}
            </div>
            <div className={styles["button-box"]}>
                <button type="button" className={buttons.cancel} onClick={() => {
                    this.props.manageEntityAccountsPrompt({});
                }}>Cancel
                </button>
                <button type="submit" className={buttons.primary}
                        disabled={!this.state.changesDone || this.props.manageSubscriptionAccountsPending}
                        onClick={this.submitButtonClicked.bind(this)}>Apply
                </button>
            </div>
            {this.props.manageEntityAccountsError && <div className={styles["error-message"]}>{this.props.manageEntityAccountsError}</div>}
        </div>);
    }
}

const mapStateToProps = state => ({
    allAccounts: state.accountsManagement.accounts,
    entityForAccountsManagement: state.accountsManagement.subscriptionForAccountsManagement,
    manageSubscriptionAccountsPending: state.accountsManagement.manageSubscriptionAccountsPending,
    manageEntityAccountsError: state.accountsManagement.manageSubscriptionAccountsError,
    type: "Subscription"
});

const mapDispatchToProps = dispatch => ({
        manageEntityAccountsPrompt: ({ entity }) => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:MANAGE-SUBSCRIPTION-ACCOUNTS-PROMPT",
                payload: { entity }
            });
        },
        manageEntityAccounts: ({ entity, accountIdsToRemove, accountIdsToAdd }) => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:MANAGE-SUBSCRIPTION-ACCOUNTS",
                payload: { subscription: entity, accountIdsToRemove, accountIdsToAdd }
            });
        }
    }
);

export default connect(mapStateToProps, mapDispatchToProps)(ManageAccountsOverlay);
