import React from 'react';
import { connect } from 'react-redux';

import { FormFieldInput } from "ui-components";
import { ReactComponent as SubscriptionIcon } from "ui-components/src/images/monochrome-icons/ic-subscription.svg";
import buttons from "ui-components/src/button/button.module.scss";
import styles from "./Accounts.module.scss";
import Loader from "../../Loader";

class RenameSubscriptionOverlay extends React.Component {
    constructor(props) {
        super(props);

        this.state = { subscriptionNewName: this.props.subscriptionForRenaming.subscriptionName };

        this.handleInputName = this.handleInputName.bind(this);
        this.renameClicked = this.renameClicked.bind(this);
    }

    handleInputName(event) {
        this.setState({ [event.target.name]: event.target.value, validationMessage: null });
    }

    renameClicked(event) {
        event.preventDefault();
        const newNamePattern = /^.{2,50}$/;

        if (newNamePattern.test(this.state.subscriptionNewName)) {
            this.props.renameSubscription({
                subscription: this.props.subscriptionForRenaming,
                newName: this.state.subscriptionNewName
            });
        } else {
            this.setState({ validationMessage: "Please enter a name that contains 1 to 50 characters." });
        }
    }


    render() {
        return (<div className={styles["rename-subscription-overlay"]}>
            {this.props.subscriptionRenamingPending && <Loader/>}
            <SubscriptionIcon/>
            <div className={styles["title"]}>Rename Subscription</div>
            <form>
                <FormFieldInput
                    type="text"
                    name="subscriptionNewName"
                    label="Subscription Name"
                    value={this.state.subscriptionNewName}
                    errorMessage={this.state.validationMessage}
                    onChange={this.handleInputName}
                    required={true}
                    autoFocus={true}
                />
                <div className={styles["button-box"]}>
                    <button type="button" className={buttons.cancel}
                            onClick={() => this.props.renameSubscriptionPrompt({})}>Cancel
                    </button>
                    <button type="submit"
                            className={buttons.primary}
                            onClick={this.renameClicked}
                            disabled={(this.props.subscriptionForRenaming.subscriptionName === this.state.subscriptionNewName) || this.props.subscriptionRenamingPending}>Rename
                    </button>
                </div>
                {this.props.subscriptionRenamingError && <div className={styles["error-message"]}>{this.props.subscriptionRenamingError}</div>}

            </form>
        </div>);
    }
}

const mapStateToProps = state => ({
    subscriptionForRenaming: state.accountsManagement.subscriptionForRenaming,
    subscriptionRenamingPending: state.accountsManagement.subscriptionRenamingPending,
    subscriptionRenamingError: state.accountsManagement.subscriptionRenamingError
});

const mapDispatchToProps = dispatch => ({
    renameSubscriptionPrompt: ({ subscription }) => {
        dispatch({
            type: "ACCOUNTS-MANAGEMENT:RENAME-SUBSCRIPTION-PROMPT",
            payload: { subscription }
        });
    },
    renameSubscription: ({ subscription, newName }) => {
        dispatch({
            type: "ACCOUNTS-MANAGEMENT:RENAME-SUBSCRIPTION",
            payload: { subscription, newName }
        });
    }
});

export default connect(mapStateToProps, mapDispatchToProps)(RenameSubscriptionOverlay);
