import React from "react";
import classNames from "classnames";
import { connect } from "react-redux";
import styles from "./Accounts.module.scss";
import { adjustPopupToBeInViewport } from "./AccountManagementUtils";

class SubscriptionPopUpMenu extends React.Component {
    constructor(props) {
        super(props);

        this.userPopUpMenu = React.createRef();
    }

    componentDidMount() {
        adjustPopupToBeInViewport(this.userPopUpMenu.current, 20);
    }

    render() {
        const subscription = this.props.entity;
        return (
            <div className={styles["popup-overlay"]} onClick={() => this.props.closePopUpMenu()}>
                <div className={classNames(styles["popup"], styles["subscription-popup-menu"])}
                     style={{ top: this.props.baseTop + (this.props.rowHeight * this.props.rowIndex) }}
                     ref={this.userPopUpMenu}>
                    <div className={styles["popup-menu-item"]}>
                        <button onClick={() => this.props.renameSubscriptionPrompt({ subscription })}>
                            Rename Subscription
                        </button>
                    </div>
                    <div className={styles["popup-menu-item"]}>
                        <button disabled={this.props.accounts.length === 1}
                                onClick={() => this.props.manageSubscriptionAccounts({ subscription })}>Manage Accounts
                        </button>
                        {this.props.accounts.length === 1 &&
                        <div className={styles["popup-menu-item-popover"]}>You only have one account</div>}
                    </div>
                </div>
            </div>
        );
    }
}

const mapStateToProps = state => ({
    accounts: state.accountsManagement.accounts
});

const mapDispatchToProps = dispatch => ({
    renameSubscriptionPrompt: ({ subscription }) => {
        dispatch({
            type: "ACCOUNTS-MANAGEMENT:RENAME-SUBSCRIPTION-PROMPT",
            payload: { subscription }
        });
    },
    manageSubscriptionAccounts: ({ subscription }) => {
        dispatch({
            type: "ACCOUNTS-MANAGEMENT:MANAGE-SUBSCRIPTION-ACCOUNTS-PROMPT",
            payload: { subscription }
        });
    }
});

export default connect(mapStateToProps, mapDispatchToProps)(SubscriptionPopUpMenu);
