import React from 'react';
import _ from "lodash";
import { connect } from 'react-redux';
import { ReactComponent as LogoAws } from 'ui-components/src/images/third-party-services/logo-aws.svg';

import styles from "./Accounts.module.scss";
import TableComponent from "./TableComponent";
import SubscriptionPopUpMenu from "./SubscriptionPopUpMenu";
import RenameSubscriptionOverlay from "./RenameSubscriptionOverlay";
import ManageAccountsOverlay from "./ManageAccountsOverlay";

const ProviderCellRenderer = props => {
    if (props.data.cloudProvider === 'aws') return <div className={styles["provider-logo"]}><LogoAws/></div>;
    return <div>{props.data.cloudProvider}</div>;
};

const rowHeight = 55; // px

const getColumnDefs = () => {
    const fields = [
        {
            headerName: 'Name',
            field: 'subscriptionName',
            tooltipField: 'subscriptionName',
            width: 450,
            cellClass: 'one-line-text',
            cellStyle: { 'line-height': `${rowHeight}px`, 'padding-left': '30px' }
        },
        { headerName: 'Service', field: 'serviceName', width: 180 },
        { headerName: 'Cloud Provider', field: 'cloudProvider', cellRenderer: 'providerCellRenderer', width: 140 },
        { headerName: 'Status', field: 'subscriptionStatus', cellStyle: { "text-transform": "capitalize" } }
    ];

    const columnDefs = fields.map(field => _.assign(field, {
        suppressMenu: true,
        unSortIcon: true,
        sortable: true,
        lockPosition: true
    }));
    columnDefs.push({
        headerName: "",
        field: "Name",
        cellRenderer: "menuCellRenderer",
        width: 80,
        cellStyle: { 'justify-content': 'flex-end', 'padding-right': '30px' },
        colId: 'button',
        lockPosition: true
    });

    return columnDefs;
};

class SubscriptionsTab extends React.Component {
    render() {
        return <div className={styles["subscriptions-tab"]}>
            {this.props.fetchSubscriptionsError &&
            <div className={styles["main-view-error-message"]}>{this.props.fetchSubscriptionsError}</div>}
            <h4>Account {this.props.account && `${this.props.account.accountName} Subscriptions`}</h4>
            {<TableComponent columnDefs={getColumnDefs()}
                             rowData={this.props.subscriptions}
                             EntityPopUpMenu={SubscriptionPopUpMenu}
                             singularTitle="Subscription"
                             pluralTitle="Subscriptions"
                             frameworkComponents={{
                                 providerCellRenderer: ProviderCellRenderer
                             }}
            />}
            {this.props.subscriptionForRenaming && <RenameSubscriptionOverlay/>}
            {this.props.subscriptionForAccountsManagement && <ManageAccountsOverlay/>}
        </div>;
    }
}

const mapStateToProps = state => ({
    account: state.accountsManagement.account,
    subscriptions: state.accountsManagement.subscriptions,
    subscriptionForRenaming: state.accountsManagement.subscriptionForRenaming,
    fetchSubscriptionsError: state.accountsManagement.fetchSubscriptionsError,
    subscriptionForAccountsManagement: state.accountsManagement.subscriptionForAccountsManagement
});

export default connect(mapStateToProps)(SubscriptionsTab);
