import React from 'react';
import _ from "lodash";
import classNames from "classnames";
import { AgGridReact } from "ag-grid-react";

import { ReactComponent as AccountAdminIcon } from "ui-components/src/images/monochrome-icons/ic-account-admin.svg";
import { ReactComponent as DotDotDotIcon } from "ui-components/src/images/monochrome-icons/ic-more.svg";

import styles from "./Accounts.module.scss";
import WorkspacesCellRenderer from "./WorkspacesCellRenderer";
import AssociatedWorkspacesPopUp from "./AssociatedWorkspacesPopUp";
import SearchWidget from "../widgets/SearchWidget";
import TableWidgetRow from "../widgets/TableWidgetRow";
import RowCounter from "../widgets/RowCounter";

const RoleCellRenderer = props => (
    <div className={styles["role-cell"]}>{props.data.userRole === 'Role-1' ?
        <span><AccountAdminIcon/>Account Admin</span> : 'Workspace Admin'}</div>
);

const MenuCellRenderer = props => (
    <div key={props.data.email} className={styles["menu-cell"]}>
        <button className={styles["inline-grid-button"]}
                onClick={() => props.context.openRowMenuPopUp({
                    data: props.data,
                    rowIndex: props.context.getRowFilteredIndex(props)
                })}>
            <DotDotDotIcon/>
        </button>
    </div>
);

const rowHeight = 55; // px
const baseTopForPopups = 290; // px

class TableComponent extends React.Component {
    constructor(props) {
        super(props);

        this.state = { gridScrollFromTop: 0, quickFilter: '' };
    }

    onFirstDataRendered(params) {
        this.api = params.api;
        params.api.sizeColumnsToFit();
    }

    onComponentStateChanged() {
        this.api.refreshCells({ force: true });
    }

    openAssociatedWorkspacesPopUp({ data, rowIndex }) {
        this.setState({ currentEntityWorkspacesPopUp: { entity: data, rowIndex } });
    }

    closeAssociatedWorkspacesPopUp() {
        this.setState({ currentEntityWorkspacesPopUp: null });
    }

    openRowMenuPopUp({ data, rowIndex }) {
        this.setState({ currentEntityMenuPopUp: { entity: data, rowIndex } });
    }

    closeRowMenuPopUp() {
        this.setState({ currentEntityMenuPopUp: null });
    }

    onGridScroll(event) {
        this.setState({ gridScrollFromTop: event.top });
    }

    onFilterChanged() {
        this.setState({ filteredCount: this.api.getModel().rootNode.childrenAfterFilter.length });
    }

    quickFilterChanged(value) {
        this.setState({ quickFilter: value });
        this.api.setQuickFilter(value);
    }

    getRowFilteredIndex(row) {
        return _.findIndex(this.api.getModel().rootNode.childrenAfterSort, { data: row.data });
    }

    render() {
        const RowPopUpMenu = this.props.EntityPopUpMenu;
        return <div>
            {this.state.currentEntityMenuPopUp &&
            <RowPopUpMenu entity={this.state.currentEntityMenuPopUp.entity}
                          rowIndex={this.state.currentEntityMenuPopUp.rowIndex}
                          rowHeight={rowHeight}
                          closePopUpMenu={this.closeRowMenuPopUp.bind(this)}
                          baseTop={baseTopForPopups - this.state.gridScrollFromTop}
            />}
            {this.state.currentEntityWorkspacesPopUp && <AssociatedWorkspacesPopUp
                workspacesList={this.state.currentEntityWorkspacesPopUp.entity.workspaces}
                rowIndex={this.state.currentEntityWorkspacesPopUp.rowIndex}
                closeAssociatedWorkspacesPopUp={this.closeAssociatedWorkspacesPopUp.bind(this)}
                rowHeight={rowHeight}
                baseTop={baseTopForPopups - this.state.gridScrollFromTop}
                left={755}
            />}
            {this.props.rowData && <TableWidgetRow>
                <RowCounter filteredCount={this.state.filteredCount} totalCount={this.props.rowData.length}
                            singularTitle={this.props.singularTitle} pluralTitle={this.props.pluralTitle}/>
                <SearchWidget setFilter={this.quickFilterChanged.bind(this)}/>
            </TableWidgetRow>}
            <div className={classNames("ag-theme-balham", "portal-ag-grid")}>
                {this.props.rowData && <AgGridReact
                    enableColResize={false}
                    columnDefs={this.props.columnDefs}
                    onBodyScroll={this.onGridScroll.bind(this)}
                    onFirstDataRendered={this.onFirstDataRendered.bind(this)}
                    onComponentStateChanged={this.onComponentStateChanged.bind(this)}
                    onFilterChanged={this.onFilterChanged.bind(this)}
                    suppressDragLeaveHidesColumns={true}
                    rowData={this.props.rowData}
                    rowHeight={rowHeight}
                    headerHeight={rowHeight}
                    suppressColumnVirtualisation="true"
                    frameworkComponents={_.assign(this.props.frameworkComponents || {}, {
                        menuCellRenderer: MenuCellRenderer,
                        roleCellRenderer: RoleCellRenderer,
                        workspacesCellRenderer: WorkspacesCellRenderer
                    })}
                    context={_.assign(this.props.context || {}, {
                        openRowMenuPopUp: this.openRowMenuPopUp.bind(this),
                        openAssociatedWorkspacesPopUp: this.openAssociatedWorkspacesPopUp.bind(this),
                        getRowFilteredIndex: this.getRowFilteredIndex.bind(this)
                    })}
                >
                </AgGridReact>}
            </div>
        </div>;
    }
}

export default TableComponent;
