import React from "react";
import classNames from "classnames";
import { connect } from "react-redux";
import styles from "./Accounts.module.scss";
import { adjustPopupToBeInViewport } from "./AccountManagementUtils";

class UserPopUpMenu extends React.Component {
    constructor(props) {
        super(props);

        this.userPopUpMenu = React.createRef();
    }

    componentDidMount() {
        adjustPopupToBeInViewport(this.userPopUpMenu.current, 20);
    }

    render() {
        const user = this.props.entity;
        return (
            <div className={styles["popup-overlay"]} onClick={() => this.props.closePopUpMenu()}>
                <div className={classNames(styles["popup"], styles["user-popup-menu"])}
                     style={{ top: this.props.baseTop + (this.props.rowHeight * this.props.rowIndex) }}
                     ref={this.userPopUpMenu}>
                    <div className={styles["popup-menu-item"]}>
                        <button disabled={user.userClientId === this.props.loggedInUserId}
                                onClick={() => this.props.disassociateUserPrompt({ user })}>Disassociate User
                        </button>
                        {user.userClientId === this.props.loggedInUserId &&
                        <div className={styles["popup-menu-item-popover"]}>You can’t remove yourself from the account.</div>}
                    </div>
                    {user.userRole === "Role-2" && <div className={styles["popup-menu-item"]}>
                        <button onClick={() => this.props.manageUserWorkspacesPrompt({ user })}>Manage Workspaces</button>
                    </div>}
                </div>
            </div>
        );
    }
}

const mapStateToProps = state => ({
    loggedInUserId: state.auth.userId
});

const mapDispatchToProps = dispatch => ({
    disassociateUserPrompt: ({ user }) => {
        dispatch({
            type: "ACCOUNTS-MANAGEMENT:DISASSOCIATE-USER-PROMPT",
            payload: user
        });
    },
    manageUserWorkspacesPrompt: ({ user }) => {
        dispatch({
            type: "ACCOUNTS-MANAGEMENT:MANAGE-USER-WORKSPACES-PROMPT",
            payload: user
        });
    }
});

export default connect(mapStateToProps, mapDispatchToProps)(UserPopUpMenu);
