import React from 'react';
import _ from "lodash";
import { connect } from 'react-redux';
import classNames from "classnames";

import buttons from "ui-components/src/button/button.module.scss";

import styles from "./Accounts.module.scss";
import DisassociateUserOverlay from "./DisassociateUserOverlay";
import { ManageUserWorkspacesOverlay } from "./ManageWorkspacesOverlay";
import AddUserOverlay from "./AddUserOverlay";
import TableComponent from "./TableComponent";
import UserPopUpMenu from "./UserPopUpMenu";

const rowHeight = 55; // px

const getColumnDefs = () => {
    const fields = [
        {
            headerName: 'Name',
            field: 'name',
            tooltipField: 'name',
            cellClass: 'one-line-text',
            cellStyle: { "padding-left": "30px", 'line-height': `${rowHeight}px` }
        },
        {
            headerName: 'Email',
            field: 'email',
            tooltipField: 'email',
            cellClass: 'one-line-text',
            cellStyle: { 'line-height': `${rowHeight}px` },
            width: 280
        },
        { headerName: 'Role', field: 'userRole', cellRenderer: 'roleCellRenderer', width: 170 },
        { headerName: 'Workspaces', field: 'workspaces', cellRenderer: 'workspacesCellRenderer' }
    ];

    const columnDefs = fields.map(field => _.assign(field, {
        suppressMenu: true,
        unSortIcon: true,
        sortable: true,
        lockPosition: true
    }));
    columnDefs.push({
        headerName: "",
        field: "Name",
        cellRenderer: "menuCellRenderer",
        colId: 'button',
        width: 80,
        cellStyle: { 'justify-content': 'flex-end', 'padding-right': '30px' },
        lockPosition: true
    });

    return columnDefs;
};

class UsersTab extends React.Component {
    render() {
        return <div className={styles["users-tab"]}>
            {this.props.fetchUsersAndWorkspacesError &&
            <div className={styles["main-view-error-message"]}>{this.props.fetchUsersAndWorkspacesError}</div>}
            <button className={classNames(buttons.primary, styles["associate-user-button"])}
                    onClick={() => this.props.setAddUserOverlayState({ newState: 'email' })}
                    disabled={this.props.loadingData || this.props.fetchUsersAndWorkspacesError}>+ Associate User
            </button>
            <h4>Account {this.props.account && `${this.props.account.accountName} Users`}</h4>
            {<TableComponent columnDefs={getColumnDefs()}
                             rowData={this.props.accountUsers}
                             EntityPopUpMenu={UserPopUpMenu}
                             singularTitle="User"
                             pluralTitle="Users"/>}
            {this.props.userForRemoval && <DisassociateUserOverlay/>}
            {this.props.userForWorkspacesManagement && <ManageUserWorkspacesOverlay/>}
            {this.props.addUserOverlayState && <AddUserOverlay/>}
        </div>;
    }
}

const mapStateToProps = state => ({
    account: state.accountsManagement.account,
    accountUsers: state.accountsManagement.accountUsers,
    userForRemoval: state.accountsManagement.userForRemoval,
    userForWorkspacesManagement: state.accountsManagement.userForWorkspacesManagement,
    addUserOverlayState: state.accountsManagement.addUserOverlayState,
    loadingData: state.accountsManagement.loadingData,
    fetchUsersAndWorkspacesError: state.accountsManagement.fetchUsersAndWorkspacesError
});

const mapDispatchToProps = dispatch => ({
    setAddUserOverlayState: ({ newState }) => {
        dispatch({
            type: "ACCOUNTS-MANAGEMENT:SET-ADD-USER-OVERLAY-STATE",
            payload: newState
        });
    }
});

export default connect(mapStateToProps, mapDispatchToProps)(UsersTab);
