import React from 'react';
import styles from "./Dashboard.module.scss"
import classNames from "classnames";
import ExternalLink from "./../widgets/ExternalLink"
import connect from "react-redux/es/connect/connect";
import * as actions from "../../actions";
import ActionButton from "../widgets/ActionButton";
import { openDialog } from "../dialog/Dialog";
import { ContactUsDialog } from "../dialog/ContactUsDialogs";
import {Link} from "react-router-dom";

const ServiceLink = ({status, serviceKey, text}) => {
    if(!status || status === actions.STATUS_PENDING) {
        return <span className={styles.cta}>Loading...</span>;
    } else {
        return <ExternalLink linkKey={`launchPad.services.${serviceKey}`} trackCategory="dashboards" className={styles.cta}>{text}</ExternalLink>;
    }
};

const CVOTwoButtons = ({view, data}) => {
    const ServiceLink = ({text}) => <ExternalLink linkKey="launchPad.services.occmSetupService" trackCategory="dashboards" className={styles.cta}>{text}</ExternalLink>;
    const ViewLink = ({text, className}) => <Link to="/cloud-managers" className={className || styles.cta}>{text}</Link>;

    if(view === 'start-free-trial') {
        return <ServiceLink text="Start Free Trial"/>
    } else if (view === 'create-new'){
        return <ServiceLink text="Create Cloud Manager"/>
    } else if (view === 'has-one-ready') {
        return <span className={styles["cta-span"]}>
            <ExternalLink linkKey="launchPad.services.cloudManager" href={data.occms[0].primaryCallbackUri} className={styles['cta-no-center']} trackCategory="dashboards">Go to Cloud Manager</ExternalLink>
            <ViewLink text="View List" className={styles['cta-no-center']}/>
        </span>
    } else if (view === 'has-one-initializing' || view === 'has-one-failed') {
        return <ViewLink text="View Cloud Manager"/>
    } else {
        return <ViewLink text="View Cloud Managers"/>
    }
};


const DashboardSecondaryHeaderDumb = ({title, launchPadServices, serviceKey, Icon, category, iconStyle}) => {
    return <header className={classNames(styles.header, "nav-adjusted-fixed-pane", `category-${category}`)}>
        <div className="header-container">
            <h1>{title}</h1>
            <Icon className="banner-logo" style={iconStyle}/>
            {serviceKey !== "cloudVolumesService" && serviceKey !== "cloudVolumesOntap" && <ServiceLink {... (launchPadServices[serviceKey] || {})}/>}
            {serviceKey === "cloudVolumesOntap" && <CVOTwoButtons view={launchPadServices.cloudVolumesOntap.view} data={launchPadServices.cloudVolumesOntap.data}/>}
        </div>
    </header>;
};

const mapStateToProps = state => {
    return {
        launchPadServices: state.services
    };
};

export const DashboardSecondaryHeader = connect(mapStateToProps)(DashboardSecondaryHeaderDumb);

const GettingStartedBox = ({Icon, title, text, linkKey, className}) => <ExternalLink className={className} linkKey={linkKey} newTab>
    <Icon/>
    <h4>{title}</h4>
    <p>{text}</p>
</ExternalLink>;

export const GettingStartedContent = ({ gettingStartedContent }) => <div>
    <h3>Learn How to Get Started</h3>
    <div className={classNames("grid-row")}>
        {gettingStartedContent.map(({ title, Icon, text, linkKey }) =>
            <GettingStartedBox className={classNames(styles["getting-started-box"], styles[`getting-started-box-x${gettingStartedContent.length}`])} linkKey={linkKey} Icon={Icon} title={title} key={title} text={text}/>
        )}
    </div>
</div>;

export const FooterContent = ({ footerContent, contactUsEmail, title }) => <footer className={styles.footer}><h4>{footerContent.title}</h4>
    <div>
        {footerContent.links.map(({ text, linkKey }) => <ExternalLink key={linkKey} linkKey={linkKey} newTab>
            {text}
        </ExternalLink>)}
        {contactUsEmail &&
        <ActionButton
            category="dashboards"
            label="contactUsDialog"
            onClick={(el) => openDialog(<ContactUsDialog className="contact-us-dialog"
                                                         email={contactUsEmail}
                                                         text={`For any questions, contact ${title} support at`}
                                                         noChat={true}/>, el.target)}>
            Contact Us
        </ActionButton>
        }
    </div>
</footer>;

export default ({title, Icon, productText, gettingStartedContent, footerContent, category, gettingStartedLink, serviceKey, contactUsEmail, ActivatedDashboard, CustomPane, iconStyle}) => {
    const hasGettingStartedContent = gettingStartedContent && gettingStartedContent.length > 0;
    const hasFooterContent = footerContent && footerContent.links.length > 0;

    return <div className={styles.base}>
        <DashboardSecondaryHeader category={category} title={title} serviceKey={serviceKey} Icon={Icon} iconStyle={iconStyle}/>
        {ActivatedDashboard && <ActivatedDashboard/>}
        {CustomPane && <CustomPane/>}
        {!CustomPane && <section className={styles.section}>
            {gettingStartedLink && !ActivatedDashboard && <ExternalLink className="getting-started-guide" linkKey={gettingStartedLink} newTab>Getting Started Guide</ExternalLink>}
            {!ActivatedDashboard && <p className={hasGettingStartedContent ? styles["intro-text"] : styles['no-content-intro-text']}>{productText}</p>}
            {hasGettingStartedContent && <GettingStartedContent gettingStartedContent={gettingStartedContent}/>}
        </section>}
        { hasFooterContent && <FooterContent footerContent={footerContent} contactUsEmail={contactUsEmail} title={title}/>}
    </div>
};