import React from 'react'
import styles from "./DeveloperHub.module.scss";
import layout from "ui-components/src/layout/layout.module.scss";
import ExternalLink from "../widgets/ExternalLink";
import CloudCentralSecondaryHeader from "../headers/CloudCentralSecondaryHeader";
import connect from "react-redux/es/connect/connect";
import {Dialog} from "./../dialog/Dialog";
import AuthenticationDialog from "./AuthenticationDialog";
import classNames from "classnames";


const DocCard = ({title, description, linkKey}) =>
    <div className={styles["doc-card"]}>
        <h4>{title}</h4>
        <p>{description}</p>
        <ExternalLink linkKey={linkKey} newTab>View API Documentation</ExternalLink>
    </div>;

const DeveloperHub = ({openAuthenticationDialog, closeAuthenticationDialog, authenticationDialogState}) => {
    let buttonRef = React.createRef();

    return <React.Fragment>
        <CloudCentralSecondaryHeader title="API Documentation">
            <button className="transparent" ref={buttonRef} onClick={openAuthenticationDialog} style={{position: "absolute", bottom: "10px", right: "0", fontSize: "12px", color: "#fff", fontWeight: "bolder"}}>
                Learn how to authenticate
            </button>
        </CloudCentralSecondaryHeader>
        <div className={classNames(styles.base, layout["main-content-container"])} style={{position: "relative"}}>
            <DocCard
                title="NetApp Cloud Central"
                description="Integrated hub for development information, help, documentation and launching services from NetApp Cloud Central."
                linkKey="developerHub.cloudCentralApiDoc"
            />

            <DocCard
                title="Cloud Volumes Service for AWS"
                description="A cloud native file service that provides NAS volumes over NFS and SMB with all-flash performance."
                linkKey="developerHub.cloudVolumesServiceAwsApiDoc"
            />

            <DocCard
                title="Cloud Volumes ONTAP"
                description="A powerful, cost-effective and easy-to-use solution for enterprise cloud storage."
                linkKey="developerHub.ontapCloudApiDoc"
            />

            <DocCard
                title="Cloud Sync"
                description="NetApp's service for rapid and secure data synchronization."
                linkKey="developerHub.cloudSyncApiDoc"
            />
        </div>

        <Dialog
            target={() => buttonRef.current}
            open={authenticationDialogState !== "CLOSED"}
            closeTrigger={closeAuthenticationDialog}
        >
            <AuthenticationDialog className={styles["authentication-dialog"]}/>
        </Dialog>

    </React.Fragment>};

const mapStateToProps = state => {
    return {
        authenticationDialogState: state.developerHub.authenticationDialogState
    };
};

const mapDispatchToProps = dispatch => {
    return {
        openAuthenticationDialog: () => {
            dispatch({
                type: "DEVELOPER-HUB:OPEN-DIALOG"
            })
        },
        closeAuthenticationDialog: () => {
            dispatch({
                type: "DEVELOPER-HUB:CLOSE-DIALOG"
            })
        }
    }
};

export default connect(mapStateToProps, mapDispatchToProps)(DeveloperHub)