import React, { Component } from 'react';
import './HeaderWidgets.scss'
import axios from 'axios'
import classNames from "classnames"
import ReactCSSTransitionGroup from 'react-addons-css-transition-group'
import CommunityMenuContent from 'components/community-menu/CommunityMenuContent'
import UserMenuContent from 'components/user-menu/UserMenuContent'
import NotificationMenuContent from 'components/notification-menu/NotificationMenuContent'
import querystring from 'query-string'
import { history } from '../../store'
import { ReactComponent as BellWithNumber} from 'images/icons/notifications/bell-with-number.svg'
import { ReactComponent as Bell0} from 'images/icons/notifications/bell-0.svg'
import { connect } from 'react-redux'
import * as actions from "./../../actions";
import ClickAway from "./../functional/ClickAway";
import BackToMyServicesLink from "../launch-pad/BackToMyServicesLink";
import { ReactComponent as CommunityIcon } from './../../images/icons/header/ic-community.svg';

const parentHasClass = (node, className) => {
    return (node.classList && node.classList.contains(className)) || (node.parentNode && parentHasClass(node.parentNode, className));
};
const fetchNotificationsTimeout = 30000;

const menuTracker = (menu, marketingMode) =>
     window.ga('send', {
        hitType: 'event',
        eventCategory: 'CloudCentral',
        eventAction: `OpenMenu${marketingMode ? 'Marketing' : ''}`,
        eventLabel: menu
    });

class HeaderWidgets extends Component {
    constructor(props){
        super(props);
        this.state = {communicationsMenuOpen: false, userMenuOpen: false, notificationMenuOpen: false};
    }

    componentWillMount(){
        const fetchNotificationsRepeat = () => {
            if(this.state.notificationMenuOpen || !this.props.auth.authenticated) { //do not update notifications while menu is open. In future merge them
                setTimeout(fetchNotificationsRepeat, fetchNotificationsTimeout);
                return;
            }

            this.props.fetchNotifications().then(() => {
                setTimeout(fetchNotificationsRepeat, fetchNotificationsTimeout);
            }, () => {
                setTimeout(fetchNotificationsRepeat, fetchNotificationsTimeout);
            })
        };

        fetchNotificationsRepeat();
        const queryParams = querystring.parse(window.location.search);
        if(queryParams.userMenuOpen !== undefined) {
            if(queryParams.userMenuOpen !== 'false') {
                this.openUserMenu();
            }

            delete queryParams.userMenuOpen;
            const searchString = querystring.stringify(queryParams);
            history.replace(window.location.pathname + (searchString ? "?" + searchString : ''));
        }
    }

    openCommunityMenu() {
        const newVal = !this.state.communicationsMenuOpen;
        if(newVal){
            menuTracker("community", this.props.marketingMode);
        }
        this.setState({communicationsMenuOpen: newVal});

        if(this.props.marketingMode){
            window.parent.postMessage({message: newVal ? "open-menu" : "close-menu"}, "*");
        }
    }


    openNotificationMenu() {
        const newVal = !this.state.notificationMenuOpen;
        if(newVal){
            menuTracker("notifications", this.props.marketingMode);
        }
        this.setState({notificationMenuOpen: newVal});

        if(this.props.marketingMode){
            window.parent.postMessage({message: newVal ? "open-menu" : "close-menu"}, "*");
        }
    }

    openUserMenu() {
        const newVal = !this.state.userMenuOpen;
        if(newVal){
            menuTracker("users", this.props.marketingMode);
        }
        this.setState({userMenuOpen: newVal});

        if(this.props.marketingMode){
            window.parent.postMessage({message: newVal ? "open-menu" : "close-menu"}, "*");
        }
    }

    render() {
        return (
            <div className="header-widgets">
                {<button className={classNames("community-button", "transparent", {"active": this.state.communicationsMenuOpen})} onClick={this.openCommunityMenu.bind(this)}>
                    <CommunityIcon/>
                </button>}
                {this.props.auth.authenticated && <button className={classNames("notification-menu-button", "transparent", {"active": this.state.notificationMenuOpen})} onClick={this.openNotificationMenu.bind(this)}>
                    <NotificationMenuIconChooser notifications={this.props.notifications} />
                </button>}
                {this.props.auth.authenticated &&
                <button className={classNames("edit-user-button", {"active": this.state.userMenuOpen})} onClick={this.openUserMenu.bind(this)} style={{whiteSpace: "nowrap"}}>
                    <span className="text-ellipsis" style={{display: "inline-block", maxWidth: "80px"}}>{this.props.auth.firstName}</span>
                    <span className="arrow">
                        <span className="lefty"/>
                        <span className="righty"/>
                    </span>
                </button>}

                <BackToMyServicesLink marketingMode={this.props.marketingMode}/>

                {!this.props.auth.authenticated && <div className="login-signup-container">
                    {!this.props.marketingMode && <span>
                        <button className="login-signup" onClick={() => window.auth.loginRedirect({}, {service: "portal", initialScreen: "login"})}>LOGIN</button>
                        <div className="divider"/>
                        <button className="login-signup" onClick={() => window.auth.loginRedirect({}, {service: "portal", initialScreen: "signUp"})}>SIGN UP</button>
                    </span>}

                    {this.props.marketingMode && <span>
                        <a className="button login-signup" href="/redirect-to-login?startOnSignup=false" target="_parent">LOGIN</a>
                        <div className="divider"/>
                        <a className="button login-signup" href="/redirect-to-login?startOnSignup=true" target="_parent">SIGN UP</a>
                    </span>}
                </div>}

                <ReactCSSTransitionGroup
                    transitionName="menu"
                    transitionEnterTimeout={400}
                    transitionLeaveTimeout={400}
                    className="community-menu-container">
                    {this.state.communicationsMenuOpen &&
                    <div className="menu-parent" >
                        <ClickAway listener = {event => {
                            if(!parentHasClass(event.target, "community-menu-container") && !parentHasClass(event.target, "community-button")){
                                this.setState({communicationsMenuOpen: false});
                                window.parent.postMessage({message: "close-menu"}, "*");
                            }
                        }}/>
                        <div key="menu" className="right-side-menu">
                            <div className="community-menu-header">
                                Community
                            </div>
                            <CommunityMenuContent />
                        </div>
                    </div>
                    }
                </ReactCSSTransitionGroup>

                <ReactCSSTransitionGroup
                    transitionName="menu"
                    transitionEnterTimeout={400}
                    transitionLeaveTimeout={400}
                    className="notification-menu-container">
                    {this.state.notificationMenuOpen && this.props.auth.authenticated &&
                    <div className="menu-parent">
                        <ClickAway listener = {
                            event => {
                                if(!parentHasClass(event.target, "notification-menu-container") && !parentHasClass(event.target, "notification-menu-button")){
                                    this.setState({notificationMenuOpen: false});
                                    window.parent.postMessage({message: "close-menu"}, "*");
                                }
                            }
                        } />
                        <div key="menu" className="right-side-menu">
                            <NotificationMenuContent/>
                        </div>
                    </div>
                    }
                </ReactCSSTransitionGroup>

                <ReactCSSTransitionGroup
                    transitionName="menu"
                    transitionEnterTimeout={400}
                    transitionLeaveTimeout={400}
                    className="user-menu-container">
                    {this.state.userMenuOpen && this.props.auth.authenticated &&
                    <div className="menu-parent">
                        <ClickAway listener = {
                            event => {
                                if(!parentHasClass(event.target, "user-menu-container") && !parentHasClass(event.target, "edit-user-button")){
                                    this.setState({userMenuOpen: false});
                                    window.parent.postMessage({message: "close-menu"}, "*");
                                }
                            }
                        }/>
                        <div key="menu" className="right-side-menu">
                            <div className="user-menu-header">
                                My Profile
                                {!this.props.marketingMode && <button className="medium primary right user-info-log-out" onClick={() => window.auth.logout()}>Log Out</button>}
                                {this.props.marketingMode && <a className="button medium primary right user-info-log-out" href="/redirect-to-login?logout=true" target="_parent">Log Out</a>}
                            </div>
                            <UserMenuContent/>
                        </div>
                    </div>
                    }
                </ReactCSSTransitionGroup>
            </div>
        )}
}

const mapDispatchToProps = dispatch => {
    return {
        fetchNotifications: () => {
            const payload = axios.get("/portal/notifications");

            payload.then(r => dispatch({type: "NOTIFICATIONS:FETCH-SUCCESS", payload: r}));

            dispatch({
                type: "NOTIFICATIONS:FETCH"
            });

            return payload;
        }
    }
};

const mapStateToProps = state => {
    return {
        auth: state.auth,
        notifications: state.notifications
    };
};

export default connect(mapStateToProps, mapDispatchToProps)(HeaderWidgets);

const NotificationMenuIconChooser = props => {
    return (
        <div>
            {(!props.notifications || props.notifications.status !== actions.STATUS_SUCCESS || props.notifications.newNotificationCount === 0) && <Bell0/>}
            {props.notifications && props.notifications.newNotificationCount > 0 && <div><BellWithNumber/></div>}
        </div>
    )
};