import React, { Component } from 'react';
import _ from 'lodash'
import './SalesToolsView.scss';
import { Link, Redirect, Route, Switch, withRouter } from 'react-router-dom';
import { connect } from 'react-redux'
import { ReactComponent as CloudVolumesLogo } from 'ui-components/src/images/netapp-services/ic-cvs.svg';
import { ReactComponent as ONTAPCloudLogo } from 'ui-components/src/images/netapp-services/ic-cvo.svg';
import { ReactComponent as CloudSyncLogo } from 'ui-components/src/images/netapp-services/ic-cloud-sync.svg';
import { ReactComponent as SaaSBackupLogo } from 'ui-components/src/images/netapp-services/ic-saas-backup.svg';
import { ReactComponent as CloudInsightsLogo } from 'ui-components/src/images/netapp-services/ic-cloud-insights.svg';
import { ReactComponent as KubernetesLogo } from 'ui-components/src/images/netapp-services/ic-kubernetes.svg';
import { ReactComponent as CloudTieringLogo } from 'ui-components/src/images/netapp-services/ic-cloud-tiering.svg';
import { ReactComponent as CloudBacupServiceLogo } from 'ui-components/src/images/netapp-services/ic-cloud-backup.svg';
import { ReactComponent as FaqIcon } from 'ui-components/src/images/2-colors-icons/faq.svg';
import { ReactComponent as TrainingIcon } from 'ui-components/src/images/2-colors-icons/ic-training.svg';
import { ReactComponent as ExclamationIcon } from 'ui-components/src/images/monochrome-icons/ic-tooltip.svg';
import { ReactComponent as DemoIcon } from 'ui-components/src/images/2-colors-icons/ic-monthly-demo.svg';
import { ReactComponent as CommissionsExplainedIcon } from 'ui-components/src/images/2-colors-icons/ic-hyperscaler-commission.svg'
import { ReactComponent as MedallionIcon } from "ui-components/src/images/2-colors-icons/ic-medallion.svg";
import { ReactComponent as ComingSoonLabel } from "ui-components/src/images/special-icons/label-new-rewards-program.svg";
import Popover from './../widgets/Popover';
import {
    CloudBackupServiceSalesTools,
    CloudInsightsSalesTools,
    CloudSyncSalesTools,
    CloudTieringSalesTools,
    CloudVolumesONTAPSalesTools,
    CloudVolumesOntapTraining,
    CloudVolumesServiceSalesTools,
    KubernetesServiceSalesTools,
    SaaSBackupSalesTools
} from './SalesToolsServiceView';
import {
    CloudVolumesServicesAwsCustomerContent,
    CloudVolumesServicesAwsHome,
    CloudVolumesServicesAwsSalesTools,
    CloudVolumesServicesAzureCustomerContent,
    CloudVolumesServicesAzureHome,
    CloudVolumesServicesAzureSalesTools,
    CloudVolumesServicesGoogleCustomerContent,
    CloudVolumesServicesGoogleHome,
    CloudVolumesServicesGoogleSalesTools
} from './CloudVolumeServiceHyperScalerView';
import { ReactComponent as PartnersIcon } from 'ui-components/src/images/2-colors-icons/ic-partner.svg';
import ExternalLink from "./../widgets/ExternalLink";
import DeveloperIntegration from "./integration-instructions/DeveloperIntegration"
import MonthlyDemosView from "./MonthlyDemosView"
import FeatureFlags from './FeatureFlags'
import { ReactComponent as IntegrationInstructionsIcon } from 'ui-components/src/images/2-colors-icons/ic-integration-instructions.svg';
import { ReactComponent as FeatureFlagsIcon } from 'ui-components/src/images/2-colors-icons/ic-feature-flags.svg';
import { ReactComponent as ConnectionManagementIcon } from 'ui-components/src/images/2-colors-icons/ic-connection.svg';
import { ReactComponent as UserManagementIcon } from 'ui-components/src/images/2-colors-icons/ic-user-manage.svg';
import CustomerSuccess from './customer-success-stories/CustomerSuccess.js'
import UserManagement from './UserManagement'
import ConnectionManagement from './ConnectionManagement'
import { registerFeature } from "../../middlewares/apiMiddleware";
import managementMiddleware from "../../middlewares/featureApis/managementMiddleware";
import internalMiddleware from "./../../middlewares/featureApis/internalMiddleware"
import customerSuccessMiddleware from "./../../middlewares/featureApis/customerSuccessMiddleware"
import querystring from "query-string";
import CloudCentralSecondaryHeader from "../headers/CloudCentralSecondaryHeader";
import AmericasGlobalSpiff from "./americas-global-spiff/AmericasGlobalSpiff";
import card from 'ui-components/src/card/card.module.scss';
import cssGrid from "ui-components/src/grid/grid.module.scss";
import layout from "ui-components/src/layout/layout.module.scss";
import classNames from "classnames";

registerFeature("MANAGEMENT", managementMiddleware);
registerFeature("INTERNAL", internalMiddleware);
registerFeature("CUSTOMER-SUCCESS", customerSuccessMiddleware);

const SalesToolsIndex = () =>
    <div className={classNames(layout["main-content-container"], "sales-tools-main", "service-view")}>
        <h4>Services</h4>
        <div className={classNames(cssGrid["col-1-sm"], cssGrid["col-2-md"], cssGrid["col-2-lg"])}>
            <Link className={card.storage} to="/netapp-internal/cloud-volumes-service">
                <CloudVolumesLogo/>
                <span>Cloud Volumes Service & Azure NetApp Files</span>
            </Link>
            <Link className={card.storage} to="/netapp-internal/cloud-volumes-ontap">
                <ONTAPCloudLogo/>
                <span>Cloud Volumes ONTAP</span>
            </Link>
            <Link className={card.control} to="/netapp-internal/kubernetes-service">
                <KubernetesLogo/>
                <span>NetApp Kubernetes Service</span>
            </Link>
            <Link className={card.analytics} to="/netapp-internal/cloud-insights">
                <CloudInsightsLogo/>
                <span>Cloud Insights</span>
            </Link>
            <Link className={card.service} to="/netapp-internal/cloud-sync">
                <CloudSyncLogo/>
                <span>Cloud Sync</span>
            </Link>
            <Link className={card.service} to="/netapp-internal/saas-backup">
                <SaaSBackupLogo/>
                <span>SaaS Backup</span>
            </Link>
            <Link className={card.service} to="/netapp-internal/cloud-tiering-service">
                <CloudTieringLogo/>
                <span>Cloud Tiering Service</span>
            </Link>
            <Link className={card.service} to="/netapp-internal/cloud-backup-service">
                <CloudBacupServiceLogo/>
                <span>Cloud Backup Service</span>
            </Link>
        </div>

        <h4>General</h4>
        <div className={classNames(cssGrid["col-1-sm"], cssGrid["col-2-md"], cssGrid["col-3-lg"])}>
            <Link className={card.card} to="/netapp-internal/americas-global-spiff" style={{position: "relative"}}>
                <ComingSoonLabel style={{position: "absolute", top: -50, left: -25}}/>
                <MedallionIcon/>
                <div>Americas & Globals Q4 CVO SPIFF</div>
            </Link>
            <ExternalLink className={card.card} linkKey="internal.sales.CDSProductCommissionsFAQLink" newTab>
                <FaqIcon/>
                <div>Commission FAQ</div>
            </ExternalLink>
            <ExternalLink className={card.card} linkKey="internal.sales.hyperscalerCommissions" newTab>
                <CommissionsExplainedIcon/>
                <div>FY19 Hyperscaler Commissions</div>
            </ExternalLink>
            <ExternalLink className={card.card} linkKey="internal.sales.cloudCentralTutorialLink" newTab>
                <TrainingIcon/>
                <div>Cloud Central Tutorial</div>
            </ExternalLink>
            <ExternalLink className={card.card} linkKey="internal.sales.cloudFirstPartnersLink" newTab>
                <PartnersIcon/>
                <div>Cloud First Partners</div>
            </ExternalLink>
            <Link className={card.card} to="/netapp-internal/monthly-demos">
                <DemoIcon/>
                <div>Monthly Demos</div>
            </Link>
        </div>

        <h4>Developers</h4>
        <div className={classNames(cssGrid["col-1-sm"], cssGrid["col-2-md"], cssGrid["col-4-lg"])}>
            <Link className={card.card} to="/netapp-internal/features">
                <FeatureFlagsIcon/>
                <div>Feature Flags</div>
            </Link>
            <Link className={card.card} to="/netapp-internal/developer-integration">
                <IntegrationInstructionsIcon/>
                <div>Developer Integration</div>
            </Link>
            <Link className={card.card} to="/netapp-internal/connection-management">
                <ConnectionManagementIcon/>
                <div>Connection Management</div>
            </Link>
            <Link className={card.card} to="/netapp-internal/user-management">
                <UserManagementIcon/>
                <div>User Management</div>
            </Link>
        </div>
    </div>
;


class MouseOverPopover extends Component {
    constructor(props) {
        super(props);
        this.state = {open: !this.props.persisted.internalPageViewed}
    }

    componentWillMount() {
        if (!this.props.persisted.internalPageViewed) {
            const eventListener11 = () => {
                this.props.setPageToViewed();
                this.setState({open: false});
                document.body.removeEventListener("click", eventListener11)
            };
            document.body.addEventListener("click", eventListener11);
        }
    }

    componentWillUnmount() {
        if (!this.props.persisted.internalPageViewed) {
            this.props.setPageToViewed();
        }
    }

    handlePopoverOpen = () => {
        this.setState({ open: true });
    };

    handlePopoverClose = () => {
        this.setState({ open: false });
    };

    render() {
        return (
            <div style={{position: "absolute", bottom: "20px", right: 0}}>
                <Popover
                    component={<div style={{float: "right", verticalAlign: "middle", fontSize: 14, fontWeight: 600, color: "#fff", cursor: "pointer"}}><ExclamationIcon style={{width: 13, height: 13, marginRight: 7, fill: "#fff"}}/>About</div>}
                    trigger="mouseover"
                    className="netapp-internal-info-popover"
                    startOpen={!this.props.persisted.internalPageViewed}
                >
                    The content of this page is only visible to NetApp users. Here you can find information about NetApp
                    Cloud Data Services and sales enablement tools for the NetApp field that can be used to better
                    assist and guide NetApp customers in their cloud journey.
                </Popover>
            </div>
        );
    }
}


const mapDispatchToProps = dispatch => {
    return {
        setPageToViewed: () => {
            dispatch({
                type: "SET-INTERNAL-VIEWED",
            })
        }
    }
};

const mapStateToProps = state => {
    return {
        persisted: state.persisted
    };
};

const ConnectedMouseOverPopover = connect(mapStateToProps, mapDispatchToProps)(MouseOverPopover);

const InternalHeader = withRouter(({ location }) =>
    _.includes(location.pathname, "americas-global-spiff") ? <div/> :
        <CloudCentralSecondaryHeader title="NetApp Internal" smartBack={true}>
            <ConnectedMouseOverPopover/>
        </CloudCentralSecondaryHeader>)
;

export default () =>
    <React.Fragment>
        <InternalHeader/>
        <div className="internal-sales-view">
            <Switch>
                <Route path="/netapp-internal/cloud-volumes-service" exact render={() => <CloudVolumesServiceSalesTools/>}/>
                <Route path="/netapp-internal/cloud-volumes-service/azure" exact render={() => <CloudVolumesServicesAzureHome/>}/>
                <Route path="/netapp-internal/cloud-volumes-service/azure/sales-tools" exact render={({location}) =>{
                    return <CloudVolumesServicesAzureSalesTools card={querystring.parse(location.search).card}/>}}/>
                <Route path="/netapp-internal/cloud-volumes-service/azure/customer-content" exact render={({location}) =>{
                    return <CloudVolumesServicesAzureCustomerContent card={querystring.parse(location.search).card}/>}}/>
                <Route path="/netapp-internal/cloud-volumes-service/aws" exact render={() => <CloudVolumesServicesAwsHome/>}/>
                <Route path="/netapp-internal/cloud-volumes-service/aws/sales-tools" exact render={({location}) =>{
                    return <CloudVolumesServicesAwsSalesTools card={querystring.parse(location.search).card}/>}}/>
                <Route path="/netapp-internal/cloud-volumes-service/aws/customer-content" exact render={({location}) =>{
                    return <CloudVolumesServicesAwsCustomerContent card={querystring.parse(location.search).card}/>}}/>
                <Route path="/netapp-internal/cloud-volumes-service/google" exact render={() => <CloudVolumesServicesGoogleHome/>}/>
                <Route path="/netapp-internal/cloud-volumes-service/google/sales-tools" exact render={({location}) =>{
                    return <CloudVolumesServicesGoogleSalesTools card={querystring.parse(location.search).card}/>}}/>
                <Route path="/netapp-internal/cloud-volumes-service/google/customer-content" exact render={({location}) =>{
                    return <CloudVolumesServicesGoogleCustomerContent card={querystring.parse(location.search).card}/>}}/>

                <Route path="/netapp-internal/cloud-volumes-ontap" exact render={() => <CloudVolumesONTAPSalesTools/>}/>
                <Route path="/netapp-internal/cloud-sync" exact render={() => <CloudSyncSalesTools/>}/>
                <Route path="/netapp-internal/saas-backup" exact render={() => <SaaSBackupSalesTools/>}/>
                <Route path="/netapp-internal/cloud-insights" exact render={() => <CloudInsightsSalesTools/>}/>
                <Route path="/netapp-internal/kubernetes-service" exact render={() => <KubernetesServiceSalesTools/>}/>
                <Route path="/netapp-internal/cloud-backup-service" exact render={() => <CloudBackupServiceSalesTools/>}/>
                <Route path="/netapp-internal/cloud-tiering-service" exact render={() => <CloudTieringSalesTools/>}/>

                <Route path="/netapp-internal/cloud-volumes-ontap/training" exact render={() => <CloudVolumesOntapTraining/>}/>
                <Route path="/netapp-internal/monthly-demos" exact render={() => <MonthlyDemosView/>}/>
                <Route path="/netapp-internal/americas-global-spiff" render={() => <AmericasGlobalSpiff/>} />
                <Route path="/netapp-internal" exact render={() => <SalesToolsIndex/>}/>

                <Route path="/netapp-internal/customer-success" exact render={() => <CustomerSuccess/>}/>
                <Route path="/netapp-internal/features" exact render={() => <FeatureFlags/>}/>

                <Route path="/netapp-internal/developer-integration" render={() => <DeveloperIntegration/>}/>
                <Route path="/netapp-internal/connection-management" render={() => <ConnectionManagement/>}/>
                <Route path="/netapp-internal/user-management" render={() => <UserManagement/>}/>
                <Redirect to="/netapp-internal"/>
            </Switch>
        </div>
    </React.Fragment>
;

