import React from 'react'
import { connect } from 'react-redux'
import { ReactComponent as CloseXIcon } from 'ui-components/src/images/monochrome-icons/x-icon.svg';
import * as actions from "../../actions";
import DemoEnvironment from '../../documents/Demo_Environment.pdf'

const ContactUsDialog = ({ closeDialog, email = 'saas.support@netapp.com' }) => {
    const mailTo = `mailto:${email}`;
    const openIntercom = () => window.Intercom('show');
    return <div>
        <header>
            Contact Us
            <button onClick={closeDialog}><CloseXIcon/></button>
        </header>
        <div className='dialog-content'>
            <p>For any questions, contact us at <a href={mailTo}>{email}</a> or start a <span className='dialog-link' onClick={openIntercom}>chat</span> with us.</p>
            <button className="dialog-button" onClick={closeDialog}>OK</button>
        </div>
    </div>
};


const OccmDemoDialogInternal = ({username, password}) => (
    <div>
        <p className='demo-cred-title'><b>Your demo credentials:</b></p>
        <p className='field-title'>User Name</p>
        <div className='field-box'>{username}</div>
        <p className='field-title'>Password</p>
        <div className='field-box'>{password}</div>
        <a href='http://demous.tlvdemonetapp.com' target="_blank" rel="noopener noreferrer" className='dialog-button' style={{float: 'left', width: '330px'}}>START NOW</a>
    </div>
);

const OccmDemoDialog = ({ occmDemoSignUp, occmDemo, closeDialog }) => {
    return (
        <div>
            <header>
                Demo Environment
                <button onClick={closeDialog}><CloseXIcon/></button>
            </header>
            {occmDemo.status === actions.STATUS_NOT_SENT && <div className='dialog-content'>
                <h3 className='dialog-subtitle'>Welcome to the OnCommand Cloud Manager simulator environment!</h3>
                <p className='main-text'>We have a full-blown, simulated OnCommand Cloud Manager (OCCM) environment that enables you to gain experience with Cloud Manager and demo Cloud Volumes ONTAP to your customers with ease and with a greatly reduced chance of failure.</p>
                <p className='main-text'>The simulation is comprehensive and allows you to test all the functionality of Cloud Manager.</p>
                <p className='main-text'><b>Before you get started, I suggest you read this <a href={DemoEnvironment} target="_blank" rel="noopener noreferrer">FAQ</a></b></p>
                <a className='dialog-button' onClick={occmDemoSignUp} href="mailto:neat@netapp.com?subject=Add Self&body=ng-occm-demous-users">CONTINUE</a>
                <div className='note-footer'>
                    <p><b>Note:</b> When you request to join the demo environment, a pop-up window opens. This pop-up enables us to add demo users to an ng list, which we use to communicate updates and important notifications. Just press Send and complete the process.</p>
                </div>
            </div>}
            {occmDemo.status === actions.STATUS_PENDING && <div className='spinner-container'>
                <svg className="spinner" width="65px" height="65px" viewBox="0 0 66 66" xmlns="http://www.w3.org/2000/svg">
                    <circle className="path" fill="none" strokeWidth="6" strokeLinecap="round" cx="33" cy="33" r="30"/>
                </svg>
            </div>}
            {occmDemo.status === actions.STATUS_SUCCESS && <div className='dialog-content'>
                {occmDemo.data.created && <p>Your new user has been created in the Demo Environment. Please login using the username and password below. If you require assistance, contact <a href='mailto:Yaron.Haimsohn@netapp.com'>yaron.haimsohn@netapp.com</a></p>}
                {!occmDemo.data.created && <p>Our records show that you already have a user in the Demo Environment. Try to login using the username and password below. If you require assistance, contact <a href='mailto:Yaron.Haimsohn@netapp.com'>yaron.haimsohn@netapp.com</a>.</p>}
                <OccmDemoDialogInternal username={occmDemo.data.username} password={occmDemo.data.password} />
            </div>}
            {occmDemo.status === actions.STATUS_ERROR && <div className='dialog-content'>
                <p className='demo-cred-title'><b>Sorry, an error has occurred</b><br /><br /></p>
                <p>An unknown error has occurred while trying to create your Demo Environment user. Please contact <a href='mailto:Yaron.Haimsohn@netapp.com'>yaron.haimsohn@netapp.com</a> for further assistance.</p>
            </div>}
        </div>
    )
};

const mapDispatchToProps = dispatch => {
    return {
        occmDemoSignUp: () => {
            dispatch({
                type: "INTERNAL:OCCM-DEMO"
            })
        }
    }
};

const mapStateToProps = state => {
    return {
        occmDemo: state.internal.occmDemo,
    };
};

const ConnectedOccmDemoDialog =  connect(mapStateToProps, mapDispatchToProps)(OccmDemoDialog);


export { ContactUsDialog, ConnectedOccmDemoDialog }