import React, {Component} from "react";
import _ from "lodash";

const getSelectOptions = (gridData, field) => {
    return _(gridData).flatMap(row => (row[field] || '').split(',')).map(_.trim).uniq().sort().value()
};

class EnumFloatingFilter extends Component {
    constructor(props) {
        super(props);
        const rowData = props.agGridReact.props.rowData;

        if(rowData){
            this.state = {
                options: getSelectOptions(rowData, this.props.column.colDef.field),
                currentValue: ''
            }
        } else {
            this.state = {
                options: [],
                currentValue: ''
            }
        }
    }

    componentDidUpdate(prevProps, prevState, snapshot) {
        const currentRowData = this.props.agGridReact.props.rowData;
        const oldRowData = prevProps.agGridReact.props.rowData;
        if (oldRowData !== currentRowData) {
            if(currentRowData){
                this.setState({options: getSelectOptions(currentRowData, this.props.column.colDef.field)})
            } else {
                this.setState({
                    options: []
                })
            }
        }
    }

    valueChanged = (event) => {
        const buildModel = (value) => {
            if (value === '') {
                return null;
            }
            return {
                filterType: 'text',
                type: 'contains',
                filter: value,
                filterTo: null
            };
        };

        this.props.onFloatingFilterChanged({model: buildModel(event.target.value)});
    };

    onParentModelChanged(parentModel) {
        this.setState({
            currentValue: !parentModel ? '' : parentModel.filter
        });
    }

    render() {
        return (
            <select value={this.state.currentValue} onChange={this.valueChanged}>
                <option value="">Show All</option>
                {this.state.options.map(value => <option key={value} value={value.toLowerCase()}>{value}</option>)}
            </select>
        )
    }
}
export default EnumFloatingFilter;

