import React, { Component } from "react"
import classNames from "classnames"
import "./ClientManagement.scss"
import connect from "react-redux/es/connect/connect";
import Loader from "./../../../Loader";
import ExternalLink from "./../../widgets/ExternalLink"
import scrollToComponent from "react-scroll-to-component";
import store from "../../../store";
import layout from "ui-components/src/layout/layout.module.scss";

const ClientListDumb = ({clients, selectClient, selectedClientId, status}) => <div className="client-list">
    {clients.map(client => <div key={client.clientId}>
            <button className={classNames("client-list-button", {active: client.clientId === selectedClientId})} onClick={() => selectClient(client.clientId)}>{client.name}</button>
        </div>)}
</div>;

const mapDispatchToPropsClientList = dispatch => {
    return {
        selectClient: (clientId) => {
            dispatch({
                type: "DEVELOPERS:CLIENTS-MANAGEMENT-SELECT-CLIENT",
                payload: {
                    clientId
                }
            })
        }
    }
};

const mapStateToPropsClientList = state => {
    return {
        clients: state.developers.managementClients.clients,
        selectedClientId: state.developers.managementClients.selectedClientId,
        status: state.developers.managementClients.status
    };
};

const ClientList = connect(mapStateToPropsClientList, mapDispatchToPropsClientList)(ClientListDumb);

const ClientDetailsDumb = ({selectedClient, saveClient, selectedClientState, selectedClientError}) => {
    const textareaAllowedCallbackUrls = React.createRef();
    const textareaAllowedWebOrigins = React.createRef();
    const textareaAllowedLogoutUrls = React.createRef();
    const textareaAllowedCors = React.createRef();

    return <div className="client-details">
        {selectedClientState === "PENDING" && <Loader/>}
        {selectedClientState === "ERROR" && <div>Failed loading client details... {selectedClientError}</div>}
        {selectedClient && <div>
            <div className="client-details-header">
                <div className="client-detail">
                    <span className="client-detail-label">Name:</span>
                    <span className="client-detail-value">{selectedClient.name}</span>
                </div>

                <div className="client-detail">
                    <span className="client-detail-label">Domain:</span>
                    <span className="client-detail-value">{selectedClient.domain}</span>
                </div>

                <div className="client-detail">
                    <span className="client-detail-label">Display Name <span className="client-detail-label-small">(Used in login page)</span>:</span>
                    <span className="client-detail-value">{selectedClient.displayName}</span>
                </div>

                <div className="client-detail">
                    <span className="client-detail-label">Client ID:</span>
                    <span className="client-detail-value">{selectedClient.clientId}</span>
                </div>
            </div>

            <div>
                <div className="label">Allowed Callback URLs</div>
                <textarea defaultValue={selectedClient.allowedCallbackUrls} ref={textareaAllowedCallbackUrls}/>
                <div className="description">
                    <p>Note: The first value in the list should be the primary domain of your service. The redirectTo for the forgot password and verification emails are to the first entry in this list.</p>
                    After the user authenticates we will only call back to any of these URLs. You can specify multiple valid URLs by comma-separating them (typically to handle different environments like QA or testing). Make sure to specify the protocol, <code>http://</code> or <code>https://</code>, otherwise the callback may fail in some cases.
                </div>
            </div>

            <div>
                <div className="label">Allowed Web Origins</div>
                <textarea defaultValue={selectedClient.allowedWebOrigins} ref={textareaAllowedWebOrigins}/>
                <div className="description">
                    Comma-separated list of allowed origins for use with <ExternalLink linkKey="developers.clientManagement.crossOriginAuthentication">Cross-Origin Authentication</ExternalLink> and <ExternalLink linkKey="developers.clientManagement.webMessageResponseMode">web message response mode</ExternalLink>, in the form of
                    <code>&lt;scheme&gt; "://" &lt;host&gt; [ ":" &lt;port&gt; ]</code>, such as
                    https://login.mydomain.com or http://localhost:3000
                </div>
            </div>

            <div>
                <div className="label">Allowed Logout URLs</div>
                <textarea defaultValue={selectedClient.allowedLogoutUrls} ref={textareaAllowedLogoutUrls}/>
                <div className="description">
                    A set of URLs that are valid to redirect to after logout from Auth0. After a user logs out from Auth0 you can redirect them with the `returnTo` query parameter. The URL that you use in `returnTo` must be listed here. You can specify multiple valid URLs by comma-separating them. You can use the star symbol as a wildcard for subdomains ('*.google.com'). Query strings and hash information are not taken into account when validating these URLs. Read more about this at <ExternalLink linkKey="developers.clientManagement.logout">https://auth0.com/docs/logout</ExternalLink>
                </div>
            </div>

            <div>
                <div className="label">Allowed Origins (CORS)</div>
                <textarea defaultValue={selectedClient.allowedCors} ref={textareaAllowedCors}/>
                <div className="description">
                    Allowed Origins are URLs that will be allowed to make requests from JavaScript to Auth0 API (typically used with CORS). By default, all your callback URLs will be allowed. This field allows you to enter other origins if you need to. You can specify multiple valid URLs by comma-separating them or one by line, and also use wildcards at the subdomain level (e.g.: https:&#47;&#47;*.contoso.com). Query strings and hash information are not taken into account when validating these URLs.
                </div>
            </div>

            <div>
                <button className="primary save-changes" onClick={() => saveClient({clientId: selectedClient.clientId, data: {
                        allowedCallbackUrls: textareaAllowedCallbackUrls.current.value,
                        allowedWebOrigins: textareaAllowedWebOrigins.current.value,
                        allowedLogoutUrls: textareaAllowedLogoutUrls.current.value,
                        allowedCors: textareaAllowedCors.current.value
                    }})}>Save Changes</button>
            </div>
        </div>}
    </div>
};

const mapDispatchToPropsClientDetails = dispatch => {
    return {
        saveClient: ({clientId, data}) => {
            dispatch({
                type: "DEVELOPERS:CLIENTS-MANAGEMENT-SAVE-CLIENT",
                payload: {
                    clientId,
                    data
                }
            })
        }
    }
};

const mapStateToPropsClientDetails = state => {
    return {
        selectedClient: state.developers.managementClients.selectedClient,
        selectedClientState: state.developers.managementClients.selectedClientState,
        selectedClientError: state.developers.managementClients.selectedClientError
    };
};

const ClientDetails = connect(mapStateToPropsClientDetails, mapDispatchToPropsClientDetails)(ClientDetailsDumb);

const ClientDetailsUpdateMessageDumb = ({clientDetailsUpdateMessage, clientDetailsUpdateMessageType}) => {
    if(clientDetailsUpdateMessage) {
        const message = React.createRef();
        setTimeout(() => {
            scrollToComponent(message.current, { offset: -200, align: 'top', duration: 400});
        });
        return <div className={classNames("client-details-update-message", clientDetailsUpdateMessageType)} ref={message}>{clientDetailsUpdateMessage}</div>
    } else {
        return null;
    }
};

const mapStateToPropsClientDetailsUpdateMessage = state => {
    return {
        clientDetailsUpdateMessage: state.developers.managementClients.clientDetailsUpdateMessage,
        clientDetailsUpdateMessageType: state.developers.managementClients.clientDetailsUpdateMessageType
    };
};

const ClientDetailsUpdateMessage = connect(mapStateToPropsClientDetailsUpdateMessage)(ClientDetailsUpdateMessageDumb);

const ClientManagementViewDumb = ({status, errorMessage}) => <div className={classNames(layout["main-content-container"], "sales-tools-main", "service-view", "client-management")}>
    <h4 className="service-title">Client Management</h4>
    {status === "PENDING" && <Loader/>}

    {status === "ERROR" && <div>
        An error occurred fetching the clients. {errorMessage}
    </div>}

    {status === "NO-CLIENTS" && <div>
        <div>
            <p>Your user does not have permissions to edit any clients.</p>
            <p>If you should be managing clients for a specific service, contact the admin of that service to request permissions over the clients.</p>
        </div>
    </div>}

    {status === "SUCCESS" && <div className="client-management-container">
        <ClientList/>
        <ClientDetails/>
        <ClientDetailsUpdateMessage/>
    </div>}
</div>;

const mapStateToProps = state => {
    return {
        status: state.developers.managementClients.status,
        clients: state.developers.managementClients.clients,
        errorMessage: state.developers.managementClients.errorMessage
    };
};

const ClientManagementView = connect(mapStateToProps)(ClientManagementViewDumb);

class ClientManagementParent extends Component {
    constructor(props){
        super(props);

        store.dispatch({
            type: "DEVELOPERS:FETCH-CLIENTS-LIST"
        });
    }

    render(){
        return <ClientManagementView/>
    }
}

export default ClientManagementParent;