import React from "react";
import {Link, Redirect, Route, Switch} from "react-router-dom";
import { ReactComponent as ContactUsIcon } from 'ui-components/src/images/2-colors-icons/contact-us.svg';
import { ReactComponent as IntegrationInstructionsIcon } from 'ui-components/src/images/2-colors-icons/ic-integration-instructions.svg';
import { ReactComponent as ClientManagementIcon } from 'ui-components/src/images/2-colors-icons/ic-client-manage.svg';
import { ReactComponent as MailingListIcon } from 'ui-components/src/images/2-colors-icons/ic-mailing-list.svg';
import { ReactComponent as StyleGuideIcon } from 'ui-components/src/images/2-colors-icons/ic-style-guide.svg';
import { ContactUsDialog } from "./../../dialog/ContactUsDialogs";
import {openDialog} from "../../dialog/Dialog";
import ExternalLink from "./../../widgets/ExternalLink";

import IntegrationInstructions from "./IntegrationInstructions";
import ClientManagement from "./ClientManagement";

import card from 'ui-components/src/card/card.module.scss';
import cssGrid from "ui-components/src/grid/grid.module.scss";
import layout from "ui-components/src/layout/layout.module.scss";
import classNames from "classnames";

import {registerFeature} from './../../../middlewares/apiMiddleware';
import developersMiddleware from "./../../../middlewares/featureApis/developersMiddleware";
registerFeature("DEVELOPERS", developersMiddleware);

const Main = () => <div className={layout["main-content-container"]}>
    <h4 className="service-title">Developer Integration</h4>

    <div className={classNames(cssGrid["col-1-sm"], cssGrid["col-2-md"], cssGrid["col-3-lg"])}>
        <Link className={card.card} to="/netapp-internal/developer-integration/integration-instructions">
            <IntegrationInstructionsIcon/>
            <div>Integration Instructions</div>
        </Link>
        <ExternalLink className={card.card} linkKey="developers.storybook">
            <StyleGuideIcon/>
            <div>Style Guide</div>
        </ExternalLink>
        <button className={card.card} onClick={(el) => openDialog(<ContactUsDialog className="contact-us-dialog" email="ng-cloud-central-dev@netapp.com" noChat={true}/>, el.target)}>
            <ContactUsIcon/>
            <div>Contact Us</div>
        </button>
        <Link className={card.card} to="/netapp-internal/developer-integration/client-management">
            <ClientManagementIcon/>
            <div>Client Management</div>
        </Link>
        <ExternalLink className={card.card} linkKey="developers.registerToDistributionList">
            <MailingListIcon/>
            <div>Subscribe to Mailing List</div>
        </ExternalLink>
    </div>
</div>;

export default () => <Switch>
    <Route path="/netapp-internal/developer-integration" exact render={() => <Main/>}/>
    <Route path="/netapp-internal/developer-integration/integration-instructions" exact render={() => <IntegrationInstructions/>}/>
    <Route path="/netapp-internal/developer-integration/client-management" exact render={() => <ClientManagement/>}/>
    <Redirect to="/netapp-internal/developer-integration"/>
</Switch>
