import React from "react"
import { ReactComponent as RightChevron } from 'ui-components/src/images/monochrome-icons/bullet-chevron-icon.svg';
import authenticationMd from "./AUTHENTICATION_README.md";
import tokenInfoReadme from "./TOKEN_INFO_README.md";
import widgetReadme from "./WIDGET_README.md";
import { ContactUsDialog } from "./../../dialog/ContactUsDialogs";
import {openDialog} from "../../dialog/Dialog";
import { connect } from 'react-redux'
import classNames from 'classnames';
import {Link} from "react-router-dom";
import Loadable from 'react-loadable';
import axios from "axios";
import {markdownRenderers} from "../markdownHelpers";
import ReactMarkdown from "react-markdown";
import layout from "ui-components/src/layout/layout.module.scss";
const LoadingView = () => <div/>;

const InformationBoxDumb = ({title, children, openRow, toggleRow}) => <div className={classNames("integration-information-box", {open: openRow === title})}>
    <h5>{title}</h5>
    <button className="transparent toggle" onClick={() => toggleRow(title)}><RightChevron/></button>
    <div className="content">
        {children}
    </div>
</div>;

const mapStateToProps = state => {
    return {
        openRow: state.internal.integrationInstructionsOpenRow,
    };
};

const mapDispatchToProps = dispatch => {
    return {
        toggleRow: (rowTitle) => {
            dispatch({
                type: "INTERNAL:INTEGRATION-INSTRUCTIONS-TOGGLE-ROW",
                payload: rowTitle
            })
        }
    }
};

const InformationBox = connect(mapStateToProps, mapDispatchToProps)(InformationBoxDumb);

const List = ({children}) => <ul>
    {children.map((child, i) => <li key={i}><RightChevron/>{child}</li>)}
</ul>;

const readmeLoader = (file) => Loadable({
    loader: ()  => axios.get(file, {baseURL: "/"}),
    render: ({data}) => {
        return <ReactMarkdown className="instructions" source={data} renderers={markdownRenderers}/>;
    },
    loading: LoadingView
});

const Authentication = readmeLoader(authenticationMd);
const Widget = readmeLoader(widgetReadme);
const TokenInfo = readmeLoader(tokenInfoReadme);

export default () => <div className={classNames(layout["main-content-container"], "sales-tools-main", "service-view")}>
    <h4 className="service-title">Integration Instructions</h4>

    <InformationBox title="Introduction">
        <p>Here you can find information on how to integrate a new service with Cloud Central</p>

        <h6>Integration involves:</h6>
        <List>
            <React.Fragment><button className="a" onClick={(el) => openDialog(<ContactUsDialog className="contact-us-dialog" text="To initialize the on-boarding process, email us at" email="ng-cloud-central-dev@netapp.com" noChat={true}/>, el.target)}>Contacting us to initialize the on-boarding process</button></React.Fragment>
            <React.Fragment>Co-ordinating with us to help with adding your service to My Services and the Widget</React.Fragment>
            <React.Fragment>Integrating with us into our staging and production environments.</React.Fragment>
            <React.Fragment>Embedding the Widget into the UI of your service</React.Fragment>
            <React.Fragment>Integrating with our authentication system so that all services use the same login functionality and have the same user database</React.Fragment>
        </List>
    </InformationBox>


    <InformationBox title="On-Boarding">
        <p>Currently the on-boarding process of integrating with Cloud Central is a manual process, and contact is done over email. We need certain details for each service, and we will create entities on our end and provide you with values.</p>

        <h6>What we need from you (can be provided as they become known):</h6>
        <List>
            <React.Fragment>Name of service</React.Fragment>
            <React.Fragment>Estimated time frames for deployments. (staging/production, GA, preview) etc...</React.Fragment>
            <React.Fragment>Domain names for staging and production endpoints</React.Fragment>
            <React.Fragment>Application type - webapp/single-page-application</React.Fragment>
        </List>

        <h6>What we will provide you:</h6>
        <List>
            <React.Fragment>OAuth ClientId/ClientSecret - for authentication purposes</React.Fragment>
            <React.Fragment>serviceKey - for use in widget</React.Fragment>
            <React.Fragment>Service Icon - as an SVG, used in our marketing</React.Fragment>
        </List>
    </InformationBox>

    <InformationBox title="Authentication">
        <Authentication/>
    </InformationBox>

    <InformationBox title="Client Management">
        <p>OAuth Clients (provided by Auth0) are integral to the authentication flow. Each service has its own client. Auth0 refers to these as Applications, more can be read about them <a href="https://auth0.com/docs/applications">here</a></p>

        <p>For single page applications and web applications you need to configure callback URLS on your client. This process can be done via the <Link to="/netapp-internal/developer-integration/client-management">Client Management Page</Link></p>

        <p>In order to be able to view clients in the Client Management page, you must have permissions those clients. During on-boarding, let us know which users can edit your clients and we will grant them permissions.</p>

        <p>Client values must be edited separately for each environment.</p>
    </InformationBox>

    <InformationBox title="Token Information">
        <p>When performing a log in either via the UI or the API (via the oauth/token endpoint), a successful login request will return with a JWT access token, and in the case that an ID token is specified, a JWT id token will also be returned. The tokens are encoded in Base 64. To view the contents of a token for debugging/development purposes, you can go to <a href="https://jwt.io/">https://jwt.io/</a> Both the access token and id token contain default claims as part of the Open IDConnect/OAuth standards as well as custom claims that we have added. Below is an explanation of these claims.</p>
        <TokenInfo/>
    </InformationBox>

    <InformationBox title="My Services Link">
        <p>Within <Link to="/my-services">My Services</Link> there is a link to each on-boarded service.</p>
        <h6>What is shown for each service</h6>
        <List>
            <React.Fragment>Title of the service</React.Fragment>
            <React.Fragment>Icon of the service</React.Fragment>
            <React.Fragment>ServiceLink - a link to the service (if the service is not GA it can be a link to register for preview)</React.Fragment>
            <React.Fragment>MarketingLink - a link to the marketing pages of the service in Cloud Central</React.Fragment>
        </List>
        <p>We support separate links for both staging and production environments if applicable.</p>

        <h6>Displaying extra information or logically changing links:</h6>
        <p>It is possible to display additional information related to the current logged in user in My Services, or to change which the target of the ServiceLink.</p>

        <List>
            <React.Fragment>An API to retrieve the information must be accessible from Cloud Central's UI</React.Fragment>
            <React.Fragment>The API should be authenticated using the standard token method as described in the authentication section</React.Fragment>
        </List>
    </InformationBox>

    <InformationBox title="Widget">
        <Widget/>
    </InformationBox>
</div>;

