import React from 'react'
import { connect } from 'react-redux'
import className from "classnames";
import * as actions from "./../../actions";
import "./RemoveOccmDialog.scss"
import { ReactComponent as ONTAPCloudLogo } from "ui-components/src/images/netapp-services/ic-cvo.svg"
import buttons from "ui-components/src/button/button.module.scss";
import dialogs from "ui-components/src/dialog/dialog.module.scss";


const DumbRemoveOccmDialog = ({closeDialog, removeOccmState, occm, removeOccm}) => {
    const wrappedRemove = () => {
        removeOccm(occm.occm)
    };
    return (<div>
            <h3>Remove Cloud Manager Listing</h3>
            <div className={className("dialog-content", {error: removeOccmState.status === actions.STATUS_ERROR})}>
                <p>Are you sure you want to remove this Cloud Manager listing?</p>
                <p>The Cloud Manager will be removed from your list of Cloud Managers, but the instance itself will remain.</p>
                <div className="occm-row">
                    <div className="logo"><ONTAPCloudLogo/></div>
                    <div className="info" style={{width: occm.primaryCallbackUri ? "180px" : "385px"}}>
                        <div className="title">Name:</div>
                        <div className="text text-ellipsis">{occm.occmName}</div>
                    </div>
                    {occm.primaryCallbackUri && <div className="info" style={{width: "180px"}}>
                        <div className="title">Address:</div>
                        <div className="text text-ellipsis">{occm.primaryCallbackUri}</div>
                    </div>}
                </div>
            </div>
            {removeOccmState.status === actions.STATUS_ERROR &&
            <div className="error-text">
                Failed to remove: {removeOccmState.data.message}
            </div>
            }
            <div className={dialogs["button-pair-wrapper"]}>
                <button className={buttons.primary} onClick={wrappedRemove} disabled={removeOccmState.status === actions.STATUS_PENDING}>{removeOccmState.status === actions.STATUS_PENDING ? "Removing..." : "Remove"}</button>
                <button className={buttons.cancel} onClick={closeDialog} disabled={removeOccmState.status === actions.STATUS_PENDING}>Cancel</button>
            </div>
        </div>
    )
};


const mapDispatchToProps = dispatch => {
    return {
        removeOccm: (occmId) => {
            dispatch({
                type: "LAUNCHPAD:REMOVE-OCCM",
                payload: {
                    occmId
                }
            });
        }
    }
};

const mapStateToProps = state => {
    return {
        removeOccmState: state.services.removeOccmState || {}
    };
};

export default connect(mapStateToProps, mapDispatchToProps)(DumbRemoveOccmDialog);