import React from 'react';
import "./SelectCloudVolumesDialog.scss"
import { ReactComponent as AzureLogo } from 'ui-components/src/images/third-party-services/logo-azure.svg';
import { ReactComponent as GoogleCloudLogo } from 'ui-components/src/images/third-party-services/logo-google-cloud.svg';
import { ReactComponent as AWSLogo } from 'ui-components/src/images/third-party-services/logo-aws.svg';
import { ReactComponent as CloudVolumesLogo } from 'ui-components/src/images/netapp-services/ic-cvs.svg';
import { CloseIcon } from 'components/svg-icons/Icons';
import { connect } from 'react-redux'
import ExternalLink from './../widgets/ExternalLink'
import * as actions from "../../actions";

const ServiceLink  = ({Logo, serviceKey, img, svgStyle, isLoading, trackCategory}) =>
    <div className="box">
        <ExternalLink linkKey={isLoading ? "" : `launchPad.services.${serviceKey}`} trackCategory={trackCategory}>
            <Logo style={svgStyle}/>
            {isLoading && <div className="is-loading">Loading...</div>}
        </ExternalLink>
        {!isLoading && <div className="the-thin-blue-line"/>}
    </div>
;

const Dialog = ({closeDialog, cloudVolumesServiceAwsUserData, trackCategory}) => {
    return (<div>
            <CloudVolumesLogo className="cloud-volumes-logo" />
            <button className="close-button" onClick={closeDialog}>
                <CloseIcon/>
            </button>
            <h3>Please select a cloud provider</h3>
            <div className="dialog-content">
                <ServiceLink trackCategory={trackCategory} serviceKey="cloudVolumesServiceAzurePreview" Logo={AzureLogo} svgStyle={{position: "relative", top: 2, height: 31}}/>
                <ServiceLink trackCategory={trackCategory} serviceKey="cloudVolumesServiceGooglePreview" Logo={GoogleCloudLogo} svgStyle={{position: "relative", top: 2}}/>
                <ServiceLink trackCategory={trackCategory} serviceKey={cloudVolumesServiceAwsUserData.registered ? "cloudVolumesServiceAws" : "cloudVolumesServiceAwsPreview"} isLoading={cloudVolumesServiceAwsUserData.status === actions.STATUS_PENDING} Logo={AWSLogo} svgStyle={{position: "relative", top: 5}}/>
            </div>
        </div>
    )
};

const mapStateToProps = state => {
    return {
        cloudVolumesServiceAwsUserData: state.services.cloudVolumesServiceAws
    };
};

export default connect(mapStateToProps)(Dialog)
