import React from 'react';
import styles from "ui-components/src/select/select.module.scss";
import classNames from "classnames";
import Select from 'react-select';

const getSelectStyles = hasError => ({
    clearIndicator: (base, state) => ({ ...base, padding: 4, display: state.getValue().length === 1 && 'none' }),
    control: base => ({
        ...base,
        minHeight: 38,
        borderColor: hasError ? '#da0000' : base.borderColor,
        marginBottom: 6,
        marginTop: 10,
        fontSize: 14
    }),
    dropdownIndicator: base => ({ ...base, padding: 4 }),
    input: base => ({ ...base, margin: 0, padding: 0 }),
    menu: base => ({ ...base, fontSize: 13, marginTop: 0 }),
    menuList: base => ({ ...base, maxHeight: 200 }),
    placeholder: base => ({ ...base, fontSize: 14 }),
    singleValue: base => ({ ...base, maxWidth: 'calc(100% - 30px)' }),
    valueContainer: base => ({ ...base, padding: '0px 15px' })
});

export default props => {
    const { label, errorMessage, reference, className, ...restProps } = props;

    return (<div className={classNames(styles["select-container"], className)}>
        <label className={errorMessage ? styles["invalid"] : ''}>
            {label}
            <Select
                {...restProps}
                classNamePrefix="react-select-component"
                styles={getSelectStyles(errorMessage)}
                ref={reference}
            />
            <div className={styles.error}>{errorMessage}</div>
        </label>
    </div>);
};
