import React, {Component} from 'react';
import classNames from "classnames";
import ReactDOM from 'react-dom';
import "./Popover.scss";
import { ReactComponent as InfoIcon } from './info-icon.svg';

const parentHasClass = (node, className) => {
    return (node.classList && node.classList.contains(className)) || (node.parentNode && parentHasClass(node.parentNode, className));
};

const closePopover = event =>{
    if(!parentHasClass(event.target, "popover-trigger-container") && !parentHasClass(event.target, "popover-container")) popoverContainer.setActivePopover(null);
};

const updatePosition = () => {
    if(!popoverContainer || !popoverContainer.state.activePopover) return;
    const trigger = ReactDOM.findDOMNode(popoverContainer.state.activePopover);
    const rect = trigger.getBoundingClientRect();
    const newState = {left: rect.left + rect.width/2 + popoverContainer.state.activePopover.state.leftAdjust, top: rect.top + rect.height + popoverContainer.state.activePopover.state.topAdjust};
    if(popoverContainer.state.left !== newState.left || popoverContainer.state.top !== newState.top) {
        popoverContainer.setState(Object.assign({}, popoverContainer.state, newState));
    }
};

document.body.addEventListener('click', closePopover);
window.addEventListener("resize", updatePosition);
window.addEventListener("scroll", updatePosition);
const observer = new MutationObserver(updatePosition);
observer.observe(document, { attributes: true, childList: true, characterData: true, subtree: true });


export default class Popover extends Component{
    constructor(props, context){
        super(props, context);
        this.state = {left: 0, top: 0, topAdjust: props.topAdjust || 0, leftAdjust: props.leftAdjust || 0, active: props.startOpen};

        if(props.startOpen){
            setTimeout(() => {
                popoverContainer.setActivePopover(this);
            }, 100);
        }
    }

    componentWillUnmount() {
        if(popoverContainer && this === popoverContainer.state.activePopover) popoverContainer.setActivePopover(null);
    }

    render(){
        const {component} = this.props;

        let triggerProps;

        if(this.props.trigger === "mouseover") triggerProps = { onMouseOver : () => popoverContainer.setActivePopover(this), onMouseOut : () => popoverContainer.setActivePopover(null)};
        else triggerProps = { onClick : (event) => {event.stopPropagation(); popoverContainer.setActivePopover(this);}};

        const trigger = component === "info" ? <button className="transparent" {...triggerProps}><InfoIcon/></button> : React.cloneElement(component, triggerProps);
        return (
            <span style={this.props.containerStyle} className={classNames("popover-trigger-container", {active: this.state.active}, this.props.containerClassName)} onClick={event => event.stopPropagation()}>
                {trigger}
            </span>
        );
    }
}

const PopoverContent = ({className, enter, left, top, children, style, arrowStyle, contentStyle}) => {
    return <div>
        <div className={classNames("popover", className, {enter})} style={Object.assign({left: left + "px", top: top + "px"}, style)}>
            <div className="arrow" style={arrowStyle}>

            </div>
            <div className="arrow-block">
            </div>
            <div className="content" style={contentStyle}>
                {children}
            </div>
        </div>
    </div>
};

class PopoverContainer extends Component{
    constructor(props, context){
        super(props, context);
        this.state = {};
    }

    componentWillMount(){
        popoverContainer = this;
    }

    setActivePopover(popover) {
        if(this.state.activePopover === popover) {
            return;
        }

        if(this.state.activePopover){
            this.state.activePopover.setState(Object.assign({}, this.state.activePopover.state, {active: false}));
        }

        this.setState({activePopover: null});

        if(popover) {
            popover.setState(Object.assign({}, popover.state, {active: true}));

            setTimeout(()=>{
                const trigger = ReactDOM.findDOMNode(popover);
                const rect = trigger.getBoundingClientRect();

                const newState = {left: rect.left + rect.width/2 + popover.state.leftAdjust, top: rect.top + rect.height + popover.topAdjust};
                this.setState({enter: true, left: newState.left, top: newState.top, activePopover: popover});

                setTimeout(() => {
                    this.setState({enter: false, left: newState.left, top: newState.top, activePopover: popover});
                })
            });
        }
    }

    render(){
        return (
            <div>
                {this.state.activePopover ? <PopoverContent children={this.state.activePopover.props.children} enter={this.state.enter} style={this.state.activePopover.props.style}
                                                            arrowStyle={this.state.activePopover.props.arrowStyle} contentStyle={this.state.activePopover.props.contentStyle} contentUp={this.state.activePopover.props.contentUp}
                                                            className={this.state.activePopover.props.className} left={this.state.left} top={this.state.top} /> : ""}
            </div>
        )
    }
}

let holder = document.getElementsByClassName("popover-container")[0];
let popoverContainer = null;
if(!holder){
    holder = document.createElement("div");
    holder.classList.add("popover-container");
    document.body.appendChild(holder);
}

if(!popoverContainer){
    ReactDOM.render(
        <PopoverContainer />,
        holder
    );
}