import _ from "lodash";
import axios from "axios";
import numeral from "numeral"
import { servicesByKey } from "../../components/fabric-view/fabricViewServices";
import links from "../../consts/links";

let tenancyAxios = null;

const dispatchEmptyService = (({dispatch}, service) => {
    dispatch({
        type: "FABRIC-VIEW:UPDATE-SERVICE",
        payload: {...service}
    });
    return Promise.resolve();
});

const dispatchLoadingServiceData = (({dispatch}, service) => {
    dispatch({
        type: "FABRIC-VIEW:UPDATE-SERVICE",
        payload: {...service, data: {
                azure: {
                    state: "loading"
                },
                onprem: {
                    state: "loading"
                },
                aws: {
                    state: "loading"
                },
                gcp: {
                    state: "loading"
                }
            }}
    });
});

const capacityFormater = (sizeInBytes) => {
    const string = numeral(sizeInBytes).format('0.0 ib');
    const arr = string.split(" ");
    return {
        value: arr[0],
        unit: arr[1]
    }
};

const dispatchData = ({dispatch}, {serviceKey, dataKey, capacity, value, text, hasData}) => {
    const doDispatch = () => {
        if(hasData){
            let out = {};
            if(value){
                out.value = value;
                out.unit = "";
            } else {
                out = capacityFormater(capacity);
            }
            dispatch({
                type: "FABRIC-VIEW:UPDATE-SERVICE-DATA",
                payload: {serviceKey, dataKey, data: {
                        value: out.value,
                        unit: out.unit,
                        text
                    }}
            });
        } else {
            dispatch({
                type: "FABRIC-VIEW:UPDATE-SERVICE-DATA",
                payload: {serviceKey, dataKey, data: null}
            });
        }
    };

    loadedPromise.then(loadedTime => {
        const currentTime = new Date().valueOf();

        if(currentTime - loadedTime > 4000){
            doDispatch();
        } else {
            const delay = Math.floor((Math.random() * (hasData ? 2500 : 1200)) + 300);
            setTimeout(() => {
                doDispatch();
            }, delay);
        }
    });

};

const translationMap = {
    "TB": "TiB",
    "GB": "GiB",
    "MB": "MiB",
    "KB": "KiB",
    "Byte": "B"
};
/**
 * OCCM lies and returns capacity as GB even though it is GiB, so convert before calculating with it
 */
const capacityExtractor = (extractor) => (entity) => {
    const capacity = _.get(entity, extractor);
    return numeral(`${capacity.size} ${translationMap[capacity.unit] || capacity.unit}`).value()
};

const handleCloudVolumesOntap = ({getState, dispatch}) => {
    return dispatchEmptyService({dispatch}, servicesByKey.cloudVolumesOntap);

    /*
    const dispatchNoCloudVolumesOntap = () => {
        dispatchEmptyService({dispatch}, servicesByKey.cloudVolumesOntap);
        dispatchEmptyService({dispatch}, servicesByKey.nativeServices);
    };

    return window.auth.refreshSso({audience: "https://cloudmanager.cloud.netapp.com"}).then(({accessToken}) => {
        const occms = _.get(getState(), "services.occms.data.occms");

        if(occms.length > 0){
            dispatchLoadingServiceData({getState, dispatch}, servicesByKey.cloudVolumesOntap);
            dispatchLoadingServiceData({getState, dispatch}, servicesByKey.nativeServices);

            const occmHandler = axios.create({
                baseURL: process.env.REACT_APP_OCCM_SERVICE_FORWARDER_URL,
                headers: {
                    Authorization: `Bearer ${accessToken}`,
                    "Content-Type": "application/json",
                    "X-Agent-id": `${occms[0].occm}clients`
                }
            });

            occmHandler.get("/occm/api/tenants").then(({data: tenants}) => {
                return tenants.forEach(tenant => {
                    occmHandler.get("/occm/api/working-environments", {
                        params: {
                            tenantId: tenant.publicId,
                            fields: "reservedSize"
                        }
                    }).then(({data: workingEnvironments}) => {
                        const hasAws = workingEnvironments.vsaWorkingEnvironments.length > 0;
                        const hasOnPrem = workingEnvironments.onPremWorkingEnvironments.length > 0;
                        const hasAzure = workingEnvironments.azureVsaWorkingEnvironments.length > 0;
                        const hasGcp = false;

                        const awsSize = _.sumBy(workingEnvironments.vsaWorkingEnvironments, capacityExtractor("reservedSize"));
                        const onpremSize = _.sumBy(workingEnvironments.onPremWorkingEnvironments, capacityExtractor("reservedSize"));
                        const azureSize = _.sumBy(workingEnvironments.azureVsaWorkingEnvironments, capacityExtractor("reservedSize"));
                        const gcpSize = false;

                        dispatchData({dispatch}, {serviceKey: "cloudVolumesOntap", dataKey: "aws", capacity: awsSize, hasData: hasAws, text: "Allocated Capacity"});
                        dispatchData({dispatch}, {serviceKey: "cloudVolumesOntap", dataKey: "azure", capacity: azureSize, hasData: hasAzure, text: "Allocated Capacity"});
                        dispatchData({dispatch}, {serviceKey: "cloudVolumesOntap", dataKey: "onprem", capacity: onpremSize,  hasData: hasOnPrem, text: "Allocated Capacity"});
                        dispatchData({dispatch}, {serviceKey: "cloudVolumesOntap", dataKey: "gcp", capacity: gcpSize, hasData: hasGcp, text: "Allocated Capacity"});
                    });

                    occmHandler.get("/occm/api/fabric-view/aws/ebs").then(({data: ebsVolumes}) => {
                            const ebsSize = _.sumBy(ebsVolumes, capacityExtractor("size"));
                            dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "aws", capacity: ebsSize, hasData: true, text: "Allocated Capacity"});
                        }, () =>
                            dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "aws", hasData: false})
                    );

                    occmHandler.get("/occm/api/fabric-view/azure/managed-disks").then(({data: managedDisks}) => {
                            const diskSize = _.sumBy(managedDisks, capacityExtractor("size"));
                            dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "azure", capacity: diskSize, hasData: true, text: "Allocated Capacity"});
                        }, () =>
                            dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "azure", hasData: false})
                    );

                    dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "onprem", hasData: false});
                    dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "gcp", hasData: false});
                });
            }, () => {
                dispatchData({dispatch}, {serviceKey: "cloudVolumesOntap", dataKey: "aws", hasData: false});
                dispatchData({dispatch}, {serviceKey: "cloudVolumesOntap", dataKey: "azure", hasData: false});
                dispatchData({dispatch}, {serviceKey: "cloudVolumesOntap", dataKey: "onprem", hasData: false});
                dispatchData({dispatch}, {serviceKey: "cloudVolumesOntap", dataKey: "gcp", hasData: false});
                dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "aws", hasData: false});
                dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "azure", hasData: false});
                dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "onprem", hasData: false});
                dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "gcp", hasData: false});
            });
        } else {
            dispatchNoCloudVolumesOntap();
        }

        return accessToken;
    }, e => {
        dispatchNoCloudVolumesOntap();
    });
    */
};

const handleCloudVolumesOntapDemo = ({dispatch}) => {
    setTimeout(() => {
        dispatchData({dispatch}, {serviceKey: "cloudVolumesOntap", dataKey: "aws", capacity: 1024*1024*1024*1024*14, hasData: true, text: "Allocated Capacity"});
    }, 475);

    setTimeout(() => {
        dispatchData({dispatch}, {serviceKey: "cloudVolumesOntap", dataKey: "azure", capacity: 1024*1024*1024*1024*12.1, hasData: true, text: "Allocated Capacity"});
    }, 750);

    setTimeout(() => {
        dispatchData({dispatch}, {serviceKey: "cloudVolumesOntap", dataKey: "gcp", hasData: false});
        dispatchData({dispatch}, {serviceKey: "cloudVolumesOntap", dataKey: "onprem",  hasData: false});
    }, 550);

    return dispatchLoadingServiceData({dispatch}, servicesByKey.cloudVolumesOntap);
};

const handleNative = ({getState, dispatch}) => {
    const dispatchEmpty = () => dispatchEmptyService({dispatch}, servicesByKey.nativeServices);
    return tenancyAxios.get(`${links.launchPad.services.tenancyBackend}/account`).then(({data: accounts}) => {
        if(accounts.length > 0) {
            const selectedAccount = accounts[0];

            return tenancyAxios.get(`${links.launchPad.services.agentsManagementBackend}/agent`, {
                params: {
                    account_id: selectedAccount.accountPublicId
                }
            }).then(({data: {agents}}) => {
                if(agents.length > 0){
                   dispatchLoadingServiceData({dispatch}, servicesByKey.nativeServices);

                    return cloudManagerAudienceTokenPromise.then((accessToken) => {
                        const occmAxios = axios.create({
                            baseURL: process.env.REACT_APP_CVO_API_URL,
                            headers: {
                                Authorization: `Bearer ${accessToken}`,
                                "Content-Type": "application/json"
                            }
                        });

                        const activeAgentIds = _(agents).filter({ status: "active" }).map("agentId").value();
                        const includesPartial = activeAgentIds.length !== agents.length;
                        const noAgentsActive = activeAgentIds.length === 0;
                        const extra = {includesPartial, noAgentsActive};
                        const ebsVolumesAccumulatorPromise = Promise.all(activeAgentIds.map(agentId =>
                            occmAxios.get("/occm/api/fabric-view/aws/ebs", {
                                headers: {
                                    "X-Agent-id": agentId
                                }
                            }).then(({data: ebsVolumes}) => ebsVolumes, () => [])
                        ));

                        ebsVolumesAccumulatorPromise.then((ebsVolumesAccumulation) => {
                            const ebsVolumes = _(ebsVolumesAccumulation).flatten().uniqBy("volumeId").value();

                            if(ebsVolumes.length > 0){
                                const ebsSize = _.sumBy(ebsVolumes, capacityExtractor("size"));
                                dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "aws", capacity: ebsSize, hasData: true, text: "Allocated Capacity", extra});
                            } else {
                                dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "aws", hasData: false, extra})
                            }
                        });

                        dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "azure", hasData: false});
                        dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "onprem", hasData: false});
                        dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "gcp", hasData: false});
                    });
               } else {
                   dispatchEmpty();
               }
            });
        } else {
            dispatchEmpty();
        }
    }).catch(() => {
        dispatchEmpty();
    });
};

const handleNativeDemo = ({dispatch}) => {
    setTimeout(() => {
        dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "aws", capacity: 1024*1024*1024*1024*3.2, hasData: true, text: "Allocated Capacity"});
    }, 850);

    setTimeout(() => {
        dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "azure", capacity: 1024*1024*1024*1024*18.9, hasData: true, text: "Allocated Capacity"});
    }, 978);

    setTimeout(() => {
        dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "onprem", hasData: false});
        dispatchData({dispatch}, {serviceKey: "nativeServices", dataKey: "gcp", hasData: false});
    }, 550);

    return dispatchLoadingServiceData({dispatch}, servicesByKey.nativeServices);
};

const handleTiering = ({dispatch}) => {
    const dispatchEmpty = () => dispatchEmptyService({dispatch}, servicesByKey.cloudTiering);

    return cloudManagerAudienceTokenPromise.then((accessToken) => {
        const tieringAxios = axios.create({
            baseURL: links.launchPad.services.cloudTiering,
            headers: {
                Authorization: `Bearer ${accessToken}`
            }
        });


        return tieringAxios.get("/api/statistics/account-exists").then(({data: accountExists}) => {
            if(accountExists){
                dispatchLoadingServiceData({dispatch}, servicesByKey.cloudTiering);
                return tieringAxios.get("/api/statistics").then(({data}) => {
                    if(data.tieredCapacity === 0 && data.numberOfClusters === 0 ){
                        dispatchData({dispatch}, {serviceKey: "cloudTiering", dataKey: "aws", hasData: false});
                    } else {
                        dispatchData({dispatch}, {serviceKey: "cloudTiering", dataKey: "aws", capacity: data.tieredCapacity, hasData: true, text: "Tiered Data"});
                    }
                    dispatchData({dispatch}, {serviceKey: "cloudTiering", dataKey: "azure", hasData: false});
                    dispatchData({dispatch}, {serviceKey: "cloudTiering", dataKey: "onprem", hasData: false});
                    dispatchData({dispatch}, {serviceKey: "cloudTiering", dataKey: "gcp", hasData: false});
                }, () => {
                    dispatchData({dispatch}, {serviceKey: "cloudTiering", dataKey: "aws", hasData: false});
                    dispatchData({dispatch}, {serviceKey: "cloudTiering", dataKey: "azure", hasData: false});
                    dispatchData({dispatch}, {serviceKey: "cloudTiering", dataKey: "onprem", hasData: false});
                    dispatchData({dispatch}, {serviceKey: "cloudTiering", dataKey: "gcp", hasData: false});
                });
            } else {
                dispatchEmpty();
            }
        }, () => {
            dispatchEmpty();
        });

    }, () => dispatchEmpty());
};

const handleTieringDemo = ({dispatch}) => {
    setTimeout(() => {
        dispatchData({dispatch}, {serviceKey: "cloudTiering", dataKey: "aws", hasData: false});
        dispatchData({dispatch}, {serviceKey: "cloudTiering", dataKey: "onprem", value: 4, hasData: true, text: "Relationships"});
        dispatchData({dispatch}, {serviceKey: "cloudTiering", dataKey: "azure", value: 2, hasData: true, text: "Relationships"});
        dispatchData({dispatch}, {serviceKey: "cloudTiering", dataKey: "gcp", value: 1, hasData: true, text: "Relationship"});
    }, 600);

    return  dispatchLoadingServiceData({dispatch}, servicesByKey.cloudTiering);
};

const handleCloudSync = ({dispatch}) => {
    return axios.get(links.launchPad.services.cloudSyncBackend + "/api/users/info?noCreate=true").then(({data: userInfo}) => {
        if(!userInfo){
            dispatchEmptyService({dispatch}, servicesByKey.cloudSync)
        } else {
            return axios.get(links.launchPad.services.cloudSyncBackend + "/api/relationships-v2").then(({data: relationships}) => {
                dispatchLoadingServiceData({dispatch}, servicesByKey.cloudSync);
                const relationshipCounts = relationships.reduce((result, el) => {
                    const type = _.get(el, "dataBroker.type", "").toLowerCase();
                    if(result[type] === null){
                        result[type] = 1;
                    } else {
                        result[type] = result[type] + 1;
                    }

                    return result;
                }, {aws: null, onprem: null, azure: null, gcp: null});

                dispatchData({dispatch}, {serviceKey: "cloudSync", dataKey: "aws", value: relationshipCounts.aws, hasData: relationshipCounts.aws !== null, text: relationshipCounts.aws > 1 ? "Relationships" : "Relationship"});
                dispatchData({dispatch}, {serviceKey: "cloudSync", dataKey: "onprem", value: relationshipCounts.onprem, hasData: relationshipCounts.onprem !== null, text: relationshipCounts.onprem > 1 ? "Relationships" : "Relationship"});
                dispatchData({dispatch}, {serviceKey: "cloudSync", dataKey: "azure", value: relationshipCounts.azure, hasData: relationshipCounts.azure !== null, text: relationshipCounts.azure > 1 ? "Relationships" : "Relationship"});
                dispatchData({dispatch}, {serviceKey: "cloudSync", dataKey: "gcp", value: relationshipCounts.gcp, hasData: relationshipCounts.gcp !== null, text: relationshipCounts.gcp > 1 ? "Relationships" : "Relationship"});
            }, () => {
                return dispatchEmptyService({dispatch}, servicesByKey.cloudSync)
            });
        }
    }, () => {
        dispatchEmptyService({dispatch}, servicesByKey.cloudSync)
    });
};

const handleCloudSyncDemo = ({dispatch}) => {
    setTimeout(() => {
        dispatchData({dispatch}, {serviceKey: "cloudSync", dataKey: "aws", hasData: false});
        dispatchData({dispatch}, {serviceKey: "cloudSync", dataKey: "onprem", value: 4, hasData: true, text: "Relationships"});
        dispatchData({dispatch}, {serviceKey: "cloudSync", dataKey: "azure", value: 2, hasData: true, text: "Relationships"});
        dispatchData({dispatch}, {serviceKey: "cloudSync", dataKey: "gcp", value: 1, hasData: true, text: "Relationship"});
    }, 600);

    return  dispatchLoadingServiceData({dispatch}, servicesByKey.cloudSync);
};

const handleCloudVolumesService = ({dispatch}) => {
    return dispatchEmptyService({dispatch}, servicesByKey.cloudVolumesService)
};

const handleCloudVolumesServiceDemo = ({dispatch}) => {
    setTimeout(() => {
        dispatchData({dispatch}, {serviceKey: "cloudVolumesService", dataKey: "gcp", capacity: 1024*1024*1024*1024*8.4, hasData: true, text: "Allocated Capacity"});
    }, 4500);

    setTimeout(() => {
        dispatchData({dispatch}, {serviceKey: "cloudVolumesService", dataKey: "aws", capacity: 1024*1024*1024*1024*4, hasData: true, text: "Allocated Capacity"});
    }, 2570);

    setTimeout(() => {
        dispatchData({dispatch}, {serviceKey: "cloudVolumesService", dataKey: "azure", hasData: false});
        dispatchData({dispatch}, {serviceKey: "cloudVolumesService", dataKey: "onprem", hasData: false, text: "Relationships"});
    }, 300);

    return  dispatchLoadingServiceData({dispatch}, servicesByKey.cloudVolumesService);
};

const handleCloudInsights = ({dispatch}) => {
    return dispatchEmptyService({dispatch}, servicesByKey.cloudInsights)
};

const handleCloudInsightsDemo = ({dispatch}) => {
    setTimeout(() => {
        dispatchData({dispatch}, {serviceKey: "cloudInsights", dataKey: "gcp", capacity: 1024*1024*1024*1024*4.4, hasData: true, text: "Monitored Capacity"});
    }, 800);

    setTimeout(() => {
        dispatchData({dispatch}, {serviceKey: "cloudInsights", dataKey: "aws", hasData: false});
    }, 650);

    setTimeout(() => {
        dispatchData({dispatch}, {serviceKey: "cloudInsights", dataKey: "azure", capacity: 1024*1024*1024*1024*8.7, hasData: true, text: "Monitored Capacity"});
        dispatchData({dispatch}, {serviceKey: "cloudInsights", dataKey: "onprem", hasData: false});
    }, 300);

    return  dispatchLoadingServiceData({dispatch}, servicesByKey.cloudInsights);
};

const handleEmpty = (serviceName) => ({dispatch}) => {
    return dispatchEmptyService({dispatch}, servicesByKey[serviceName])
};

const handleKubernetesService = ({dispatch}) => {
    return axios.get(`${links.launchPad.services.kubernetesServiceBackend}/orgs`).then(({data: orgs}) => {
        let awsCount = 0, azureCount = 0, gcpCount = 0, onpremCount = 0;

        const clustersPromise = orgs.map(org => {
           return axios.get(`${links.launchPad.services.kubernetesServiceBackend}/orgs/${org.pk}/clusters`).then(({data: clusters}) => {
               clusters.forEach(({provider}) => {
                   if(provider === "aws"){
                       awsCount = awsCount + 1;
                   } else if (provider === "azure"){
                       azureCount = azureCount + 1;
                   } else if (provider === "gce" || provider === "gks"){
                       gcpCount = gcpCount + 1;
                   }
               });
           });
        });

        return Promise.all(clustersPromise).then(() => {
            if(awsCount === 0 && azureCount === 0 && gcpCount === 0 && onpremCount === 0){
                dispatchEmptyService({dispatch}, servicesByKey.kubernetesService);
            } else {
                dispatchLoadingServiceData({dispatch}, servicesByKey.kubernetesService);
                dispatchData({dispatch}, {serviceKey: "kubernetesService", dataKey: "aws", value: awsCount, hasData: awsCount > 0, text: awsCount > 1 ? "Clusters" : "Cluster"});
                dispatchData({dispatch}, {serviceKey: "kubernetesService", dataKey: "azure", value: azureCount, hasData: azureCount > 0, text: azureCount > 1 ? "Clusters" : "Cluster"});
                dispatchData({dispatch}, {serviceKey: "kubernetesService", dataKey: "gcp", value: gcpCount, hasData: gcpCount > 0, text: gcpCount > 1 ? "Clusters" : "Cluster"});
                dispatchData({dispatch}, {serviceKey: "kubernetesService", dataKey: "onprem", hasData: false});
            }
        }, () => {
            dispatchEmptyService({dispatch}, servicesByKey.kubernetesService);
        });
    }, () => {
        dispatchEmptyService({dispatch}, servicesByKey.kubernetesService);
    });
};

const handleKubernetesServiceDemo = ({dispatch}) => {
    setTimeout(() => {
        dispatchData({dispatch}, {serviceKey: "kubernetesService", dataKey: "gcp", value: 4, hasData: true, text: "Managed Clusters"});
        dispatchData({dispatch}, {serviceKey: "kubernetesService", dataKey: "aws", value: 3, hasData: true, text: "Managed Clusters"});
        dispatchData({dispatch}, {serviceKey: "kubernetesService", dataKey: "azure", value: 8, hasData: true, text: "Managed Clusters"});
    }, 725);

    setTimeout(() => {
        dispatchData({dispatch}, {serviceKey: "kubernetesService", dataKey: "onprem", hasData: false});
    }, 300);

    return  dispatchLoadingServiceData({dispatch}, servicesByKey.kubernetesService);
};

let loadedPromise = Promise.resolve(new Date().valueOf());
let cloudManagerAudienceTokenPromise = null;
export default ({getState, dispatch, next, action}) => {
    const { type } = action;
    const demoMode = _.get(getState(), "features.active.demoMode");

    if(demoMode) {
        if (type === "FABRIC-VIEW:INITIALIZE") {
            setTimeout(() => {
                if (type === "FABRIC-VIEW:INITIALIZE") {
                    Promise.all([handleCloudVolumesOntapDemo({dispatch}),
                        handleNativeDemo({dispatch}),
                        handleTieringDemo({dispatch}),
                        handleCloudSyncDemo({dispatch}),
                        handleCloudVolumesServiceDemo({dispatch}),
                        handleCloudInsightsDemo({dispatch}),
                        handleEmpty("saasBackup")({getState, dispatch}),
                        handleEmpty("cloudSecure")({getState, dispatch}),
                        handleEmpty("cloudBackup")({getState, dispatch}),
                        handleKubernetesServiceDemo({dispatch})
                    ]).then(() => {
                        dispatch({
                            type: "FABRIC-VIEW:ALL-LOADED"
                        });

                        const servicesCount = _.keys(servicesByKey).length;

                        _.forEach(_.range(servicesCount + 1), index => {
                            setTimeout(() => {
                                dispatch({ type: "FABRIC-VIEW:SHOW-COUNT", payload: { index: index } });
                            }, 100 * index);
                        });
                    });
                }
            }, 1000);
        }
    } else {
        if (type === "FABRIC-VIEW:INITIALIZE") {
            tenancyAxios = axios.create({
                headers: {
                    'X-User-Token': axios.defaults.headers.common['Authorization']
                }
            });

            cloudManagerAudienceTokenPromise = window.auth.refreshSso({audience: "https://cloudmanager.cloud.netapp.com"}).then(({accessToken}) => (accessToken));

            loadedPromise = Promise.all([handleCloudVolumesOntap({getState, dispatch}),
                handleNative({getState, dispatch}),
                handleTiering({getState, dispatch}),
                handleCloudSync({getState, dispatch}),
                handleCloudVolumesService({getState, dispatch}),
                handleCloudInsights({getState, dispatch}),
                handleEmpty("saasBackup")({getState, dispatch}),
                handleEmpty("cloudSecure")({getState, dispatch}),
                handleEmpty("cloudBackup")({getState, dispatch}),
                handleKubernetesService({getState, dispatch})
            ]).then(() => {
                dispatch({
                    type: "FABRIC-VIEW:ALL-LOADED"
                });

                const servicesCount = _.keys(servicesByKey).length;

                _.forEach(_.range(servicesCount + 1), index => {
                    setTimeout(() => {
                        dispatch({ type: "FABRIC-VIEW:SHOW-COUNT", payload: { index: index } });
                    }, 100 * index);
                });

                return new Date().valueOf();
            });
        }
    }

    if (type === "FABRIC-VIEW:INITIALIZE") {
        return next({
            type: "FABRIC-VIEW:INITIALIZE",
            payload: {
                servicesCount: _.keys(servicesByKey).length
            }
        })
    }

    return next(action);
};

