import _ from 'lodash';

/* eslint default-case: 0 */

const enhanceAgentsWithWorkspacesData = ({ agents, workspaces }) => _.map(agents, agent => {
    agent.workspaces = _.filter(workspaces, workspace => _.includes(workspace.associatedAgents, agent.agentId));
    return agent;
});

const defaultState = () => (
    { activeTab: 'Users', loadingData: 0 }
);

export default (state = defaultState(), { payload, type }) => {
    switch (type) {
        case "ACCOUNTS-MANAGEMENT:SET-ACCOUNT-ID": {
            return { ...state, accountId: payload.accountId };
        }
        case "ACCOUNTS-MANAGEMENT:FETCH-ACCOUNTS-PENDING": {
            return { ...state, loadingData: state.loadingData + 1, fetchAccountsError: null };
        }
        case "ACCOUNTS-MANAGEMENT:FETCH-ACCOUNTS-SUCCESS": {
            return {
                ...state,
                accounts: payload.accounts,
                account: payload.account,
                loadingData: state.loadingData - 1
            };
        }
        case "ACCOUNTS-MANAGEMENT:FETCH-ACCOUNTS-ERROR": {
            return { ...state, fetchAccountsError: payload.errorMessage, loadingData: state.loadingData - 1 };
        }
        case "ACCOUNTS-MANAGEMENT:FETCH-USERS-AND-WORKSPACES-PENDING": {
            return { ...state, loadingData: state.loadingData + 1, fetchUsersAndWorkspacesError: null };
        }
        case "ACCOUNTS-MANAGEMENT:FETCH-USERS-AND-WORKSPACES-SUCCESS": {
            return {
                ...state,
                accountUsers: payload.users,
                accountWorkspaces: payload.workspaces,
                agents: enhanceAgentsWithWorkspacesData({ agents: state.agents, workspaces: payload.workspaces }),
                loadingData: state.loadingData - 1
            };
        }
        case "ACCOUNTS-MANAGEMENT:FETCH-USERS-AND-WORKSPACES-ERROR": {
            return { ...state, fetchUsersAndWorkspacesError: payload.errorMessage, loadingData: state.loadingData - 1 };
        }
        case "ACCOUNTS-MANAGEMENT:FETCH-AGENTS-PENDING": {
            return { ...state, loadingData: state.loadingData + 1, fetchAgentsError: null };
        }
        case "ACCOUNTS-MANAGEMENT:FETCH-AGENTS-SUCCESS": {
            return {
                ...state,
                agents: enhanceAgentsWithWorkspacesData({ agents: payload.agents, workspaces: state.workspaces }),
                loadingData: state.loadingData - 1
            };
        }
        case "ACCOUNTS-MANAGEMENT:FETCH-AGENTS-ERROR": {
            return { ...state, fetchAgentsError: payload.errorMessage, loadingData: state.loadingData - 1 };
        }
        case "ACCOUNTS-MANAGEMENT:FETCH-SUBSCRIPTIONS-PENDING": {
            return { ...state, loadingData: state.loadingData + 1, fetchSubscriptionsError: null };
        }
        case "ACCOUNTS-MANAGEMENT:FETCH-SUBSCRIPTIONS-SUCCESS": {
            return {
                ...state,
                subscriptions: payload.subscriptions,
                loadingData: state.loadingData - 1
            };
        }
        case "ACCOUNTS-MANAGEMENT:FETCH-SUBSCRIPTIONS-ERROR": {
            return { ...state, fetchSubscriptionsError: payload.errorMessage, loadingData: state.loadingData - 1 };
        }
        case "ACCOUNTS-MANAGEMENT:SET-TAB": {
            return {
                ...state,
                activeTab: payload.tabName,
                addUserOverlayState: null,
                userForWorkspacesManagement: null,
                userForRemoval: null,
                agentForWorkspacesManagement: null,
                subscriptionForRenaming: null,
                subscriptionForAccountsManagement: null
            };
        }
        case "ACCOUNTS-MANAGEMENT:DISASSOCIATE-USER-PROMPT": {
            return { ...state, userForRemoval: payload, disassociateUserError: null };
        }
        case "ACCOUNTS-MANAGEMENT:DISASSOCIATE-USER-PENDING": {
            return { ...state, disassociateUserError: null, disassociateUserPending: true };
        }
        case "ACCOUNTS-MANAGEMENT:DISASSOCIATE-USER-SUCCESS": {
            return {
                ...state,
                userForRemoval: null,
                accountUsers: _.without(state.accountUsers, _.find(state.accountUsers, { userPublicId: payload.associatedUserPublicId })),
                disassociateUserPending: null
            };
        }
        case "ACCOUNTS-MANAGEMENT:DISASSOCIATE-USER-ERROR": {
            return { ...state, disassociateUserError: payload.errorMessage, disassociateUserPending: null };
        }
        case "ACCOUNTS-MANAGEMENT:MANAGE-USER-WORKSPACES-PROMPT": {
            return { ...state, userForWorkspacesManagement: payload };
        }
        case "ACCOUNTS-MANAGEMENT:MANAGE-USER-WORKSPACES-PENDING": {
            return { ...state, manageUserWorkspacesPending: true, manageUserWorkspacesError: null };
        }
        case "ACCOUNTS-MANAGEMENT:MANAGE-USER-WORKSPACES-SUCCESS": {
            return { ...state, manageUserWorkspacesPending: null, userForWorkspacesManagement: null };
        }
        case "ACCOUNTS-MANAGEMENT:MANAGE-USER-WORKSPACES-ERROR": {
            return { ...state, manageUserWorkspacesPending: null, manageUserWorkspacesError: payload.errorMessage };
        }
        case "ACCOUNTS-MANAGEMENT:EDIT-WORKSPACE-NAME-PENDING": {
            return { ...state, workspaceRenamePending: payload, manageWorkspaceError: null };
        }
        case "ACCOUNTS-MANAGEMENT:EDIT-WORKSPACE-NAME-SUCCESS": {
            return { ...state, workspaceRenamePending: null };
        }
        case "ACCOUNTS-MANAGEMENT:EDIT-WORKSPACE-NAME-ERROR": {
            return {
                ...state,
                workspaceRenamePending: null,
                manageWorkspaceError: payload.errorMessage,
                failedRenameWorkspace: payload.workspace
            };
        }
        case "ACCOUNTS-MANAGEMENT:DELETE-WORKSPACE-PENDING": {
            return {
                ...state,
                workspaceDeletePending: payload,
                manageWorkspaceError: null,
                failedRenameWorkspace: null
            };
        }
        case "ACCOUNTS-MANAGEMENT:DELETE-WORKSPACE-SUCCESS": {
            return { ...state, workspaceDeletePending: null };
        }
        case "ACCOUNTS-MANAGEMENT:DELETE-WORKSPACE-ERROR": {
            return { ...state, workspaceDeletePending: null, manageWorkspaceError: payload.errorMessage };
        }
        case "ACCOUNTS-MANAGEMENT:ADD-NEW-WORKSPACE-PENDING": {
            return { ...state, addNewWorkspacePending: true, manageWorkspaceError: null };
        }
        case "ACCOUNTS-MANAGEMENT:ADD-NEW-WORKSPACE-SUCCESS": {
            return {
                ...state,
                addNewWorkspacePending: null,
                accountWorkspaces: _.sortBy(state.accountWorkspaces.concat(payload), [workspace => workspace.workspaceName.toLowerCase()])
            };
        }
        case "ACCOUNTS-MANAGEMENT:ADD-NEW-WORKSPACE-ERROR": {
            return { ...state, addNewWorkspacePending: null, manageWorkspaceError: payload.errorMessage };
        }
        case "ACCOUNTS-MANAGEMENT:DISPLAY-WORKSPACES-ERROR": {
            return { ...state, manageWorkspaceError: payload.errorMessage };
        }
        case "ACCOUNTS-MANAGEMENT:SET-ADD-USER-OVERLAY-STATE": {
            return {
                ...state,
                addUserOverlayState: payload,
                associatedEmailInputFeedback: null,
                userForWorkspacesManagement: null,
                userForRemoval: null,
                addUserError: null
            };
        }
        case "ACCOUNTS-MANAGEMENT:CHECK-IF-ASSOCIATED-EMAIL-EXISTS-IN-CLOUD-CENTRAL-SUCCESS": {
            return {
                ...state,
                associatedEmailExistsInCloudCentral: payload,
                associatedEmailExistsInCloudCentralPending: null,
                associatedEmailInputFeedback: payload.valid ? '' : 'This Email doesn\'t exist in Cloud Central'
            };
        }
        case "ACCOUNTS-MANAGEMENT:CHECK-IF-ASSOCIATED-EMAIL-EXISTS-IN-CLOUD-CENTRAL-ERROR": {
            return {
                ...state,
                associatedEmailInputFeedback: payload.errorMessage,
                associatedEmailExistsInCloudCentralPending: null
            };
        }
        case "ACCOUNTS-MANAGEMENT:CHECK-IF-ASSOCIATED-EMAIL-EXISTS-IN-CLOUD-CENTRAL-PENDING": {
            return { ...state, associatedEmailExistsInCloudCentralPending: true };
        }
        case "ACCOUNTS-MANAGEMENT:SET-ASSOCIATED-EMAIL-INPUT-FEEDBACK": {
            return { ...state, associatedEmailInputFeedback: payload };
        }
        case "ACCOUNTS-MANAGEMENT:ASSOCIATE-USER-PENDING": {
            return { ...state, addUserPending: true, addUserError: null };
        }
        case "ACCOUNTS-MANAGEMENT:ASSOCIATE-USER-SUCCESS": {
            return { ...state, addUserOverlayState: null, addUserPending: null };
        }
        case "ACCOUNTS-MANAGEMENT:ASSOCIATE-USER-ERROR": {
            return { ...state, addUserError: payload.errorMessage, addUserPending: null };
        }
        case "ACCOUNTS-MANAGEMENT:MANAGE-AGENT-WORKSPACES-PROMPT": {
            return { ...state, agentForWorkspacesManagement: payload.agent, manageAgentWorkspacesError: null };
        }
        case "ACCOUNTS-MANAGEMENT:MANAGE-AGENT-WORKSPACES-PENDING": {
            return { ...state, manageAgentWorkspacesPending: true, manageAgentWorkspacesError: null };
        }
        case "ACCOUNTS-MANAGEMENT:MANAGE-AGENT-WORKSPACES-SUCCESS": {
            return { ...state, manageAgentWorkspacesPending: null, agentForWorkspacesManagement: null };
        }
        case "ACCOUNTS-MANAGEMENT:MANAGE-AGENT-WORKSPACES-ERROR": {
            return { ...state, manageAgentWorkspacesPending: null, manageAgentWorkspacesError: payload.errorMessage };
        }
        case "ACCOUNTS-MANAGEMENT:RENAME-SUBSCRIPTION-PROMPT": {
            return { ...state, subscriptionForRenaming: payload.subscription, subscriptionRenamingError: null };
        }
        case "ACCOUNTS-MANAGEMENT:RENAME-SUBSCRIPTION-PENDING": {
            return { ...state, subscriptionRenamingPending: true, subscriptionRenamingError: null };
        }
        case "ACCOUNTS-MANAGEMENT:RENAME-SUBSCRIPTION-SUCCESS": {
            return { ...state, subscriptionRenamingPending: null, subscriptionForRenaming: null };
        }
        case "ACCOUNTS-MANAGEMENT:RENAME-SUBSCRIPTION-ERROR": {
            return { ...state, subscriptionRenamingPending: null, subscriptionRenamingError: payload.errorMessage };
        }
        case "ACCOUNTS-MANAGEMENT:MANAGE-SUBSCRIPTION-ACCOUNTS-PROMPT": {
            return {
                ...state,
                subscriptionForAccountsManagement: payload.subscription,
                manageSubscriptionAccountsError: null
            };
        }
        case "ACCOUNTS-MANAGEMENT:MANAGE-SUBSCRIPTION-ACCOUNTS-PENDING": {
            return { ...state, manageSubscriptionAccountsPending: true, manageSubscriptionAccountsError: null };
        }
        case "ACCOUNTS-MANAGEMENT:MANAGE-SUBSCRIPTION-ACCOUNTS-SUCCESS": {
            return { ...state, manageSubscriptionAccountsPending: null, subscriptionForAccountsManagement: null };
        }
        case "ACCOUNTS-MANAGEMENT:MANAGE-SUBSCRIPTION-ACCOUNTS-ERROR": {
            return {
                ...state,
                manageSubscriptionAccountsPending: null,
                manageSubscriptionAccountsError: payload.errorMessage
            };
        }
        case "ACCOUNTS-MANAGEMENT:CLOSE-ALL-OVERLAYS": {
            return {
                ...state,
                currentUserWorkspacesPopUp: null,
                userForRemoval: null,
                userForWorkspacesManagement: null,
                addUserOverlayState: null,
                associatedEmailInputFeedback: null,
                agentForWorkspacesManagement: null,
                subscriptionForAccountsManagement: null
            };
        }
    }

    return state;
};
