import { createStore, applyMiddleware, compose } from 'redux'
import { createBrowserHistory } from 'history'
import createRootReducer from './reducers'
import persistState from 'redux-localstorage'
import authMiddleware from "./middlewares/authMiddleware"
import trackingMiddleware from "./middlewares/trackingMiddleware"
import apiMiddleware from "./middlewares/apiMiddleware"

export const history = createBrowserHistory();

const initialState = {};
const enhancers = [persistState('persisted', {key: 'NetAppCloudCentral'})];

const middleware = [
    authMiddleware,
    apiMiddleware,
    trackingMiddleware
];

const composedEnhancers = compose(
    applyMiddleware(...middleware),
    ...enhancers
);

const store = createStore(
    createRootReducer(),
    initialState,
    composedEnhancers
);

export default store